// BitFieldCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "FF4Edit.h"
#include "BitFieldCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBitFieldCtrl

CBitFieldCtrl::CBitFieldCtrl()
{
	m_nBitCount = 8;
	m_dwValue = 0;
	m_bJustGone = FALSE;
}

CBitFieldCtrl::~CBitFieldCtrl()
{
}


BEGIN_MESSAGE_MAP(CBitFieldCtrl, CButton)
	//{{AFX_MSG_MAP(CBitFieldCtrl)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_DESTROY()
	ON_WM_LBUTTONDBLCLK()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBitFieldCtrl message handlers

void CBitFieldCtrl::DrawItem(LPDRAWITEMSTRUCT lpdis) 
{
	CDC dc;  dc.Attach(lpdis->hDC);

	CRect rect = lpdis->rcItem;

	dc.DrawEdge(rect, EDGE_SUNKEN, BF_RECT);
	
	CRect innerRect = rect;  innerRect.DeflateRect(2, 2);
	innerRect.right -= ::GetSystemMetrics(SM_CXVSCROLL);
	dc.FillSolidRect(innerRect, ::GetSysColor(COLOR_WINDOW));

	CRect buttonRect = rect;  buttonRect.DeflateRect(2, 2);
	buttonRect.left = buttonRect.right - ::GetSystemMetrics(SM_CXVSCROLL);

	if(m_wndList->IsWindowVisible())
	{
		dc.DrawFrameControl(buttonRect, DFC_SCROLL, DFCS_SCROLLCOMBOBOX | DFCS_FLAT | DFCS_PUSHED);
	}
	else
	{
		dc.DrawFrameControl(buttonRect, DFC_SCROLL, DFCS_SCROLLCOMBOBOX);
	}

	CRect textRect = innerRect;  textRect.DeflateRect(2, 2);

	textRect.left += (textRect.Width() - 14 * m_nBitCount) / 2;
	textRect.top += (textRect.Height() - 14) / 2;

	for(int i = m_nBitCount - 1; i >= 0; i--)
	{
		m_bitRect[i] = CRect(textRect.left + 14 * (m_nBitCount - (i + 1)), textRect.top,
			textRect.left + 14 * (m_nBitCount - i), textRect.bottom);
		m_bitRect[i].NormalizeRect();

		if((m_dwValue >> i) & 0x1) m_images.Draw(&dc, 1, CPoint(m_bitRect[i].left, m_bitRect[i].top), ILD_NORMAL);
		else m_images.Draw(&dc, 0, CPoint(m_bitRect[i].left, m_bitRect[i].top), ILD_NORMAL);
	}

	if(lpdis->itemState & ODS_FOCUS)
	{
		CRect focusRect = CRect(m_bitRect[m_nBitCount - 1].left, m_bitRect[m_nBitCount - 1].top,
			m_bitRect[0].right - 1, m_bitRect[0].bottom);  focusRect.InflateRect(1, 1);
		dc.DrawFocusRect(focusRect);
	}

	dc.Detach();
}

void CBitFieldCtrl::SetValue(DWORD dwVal)
{
	m_dwValue = dwVal;
	Invalidate();
}

void CBitFieldCtrl::SetBitName(int nBit, CString strName)
{
	m_bitNames[nBit] = strName;
}

void CBitFieldCtrl::PreSubclassWindow() 
{
	m_images.Create(IDB_CHECKBOXES, 14, 0, RGB(255, 0, 255));
	EnableToolTips(TRUE);

	CButton::PreSubclassWindow();

	m_wndList = new CBitCtrlList;
	m_wndList->Create(this);
	m_wndList->SetFont(GetFont());
}

void CBitFieldCtrl::OnDestroy() 
{
	CButton::OnDestroy();
}

int CBitFieldCtrl::OnToolHitTest(CPoint point, TOOLINFO* pTI) const
{
	int nIndex = -1;

	for(int i = m_nBitCount - 1; i >= 0; i--)
	{
		if(m_bitRect[i].PtInRect(point))
		{
			nIndex = i;
			break;
		}
	}

	if(nIndex != -1)
	{
		pTI->lpszText = (LPTSTR)malloc(m_bitNames[nIndex].GetLength() + 1);
		lstrcpy(pTI->lpszText, m_bitNames[nIndex]);

		pTI->rect = m_bitRect[nIndex];
		pTI->uId = (UINT)GetSafeHwnd();
		pTI->hwnd = GetSafeHwnd();
	}

	return nIndex;
}

void CBitFieldCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CRect buttonRect;  GetClientRect(buttonRect);  buttonRect.DeflateRect(2, 2);
	buttonRect.left = buttonRect.right - ::GetSystemMetrics(SM_CXVSCROLL);
	
	// If the button is down in one of our checkboxes, return immediately
	// instead of passing the message along.  This allows us to suppress the
	// popup window and handle the checkbox click in OnLButtonUp().

	for(int i = 0; i < m_nBitCount; i++)
	{
		if(m_bitRect[i].PtInRect(point))
		{
			SetFocus();
			return;
		}
	}

	if(m_bJustGone)
	{
		m_bJustGone = FALSE;
		return;
	}

	if(m_wndList->IsWindowVisible())
	{
		m_wndList->CancelWindow();
		return;
	}

	CButton::OnLButtonDown(nFlags, point);

	CRect rect;  GetClientRect(rect);
	ClientToScreen(&rect);
	CPoint ptPopup(rect.left, rect.bottom);
	m_wndList->Invoke(this, ptPopup, rect.Width(), m_nBitCount, m_bitNames);
}

void CBitFieldCtrl::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	CRect buttonRect;  GetClientRect(buttonRect);  buttonRect.DeflateRect(2, 2);
	buttonRect.left = buttonRect.right - ::GetSystemMetrics(SM_CXVSCROLL);
	
	// If the button is down in one of our checkboxes, return immediately
	// instead of passing the message along.  This allows us to suppress the
	// popup window and handle the checkbox click in OnLButtonUp().

	for(int i = 0; i < m_nBitCount; i++)
	{
		if(m_bitRect[i].PtInRect(point))
		{
			SetFocus();
			return;
		}
	}

	if(m_bJustGone)
	{
		m_bJustGone = FALSE;
		return;
	}

	if(m_wndList->IsWindowVisible())
	{
		m_wndList->CancelWindow();
		return;
	}

	CButton::OnLButtonDblClk(nFlags, point);

	CRect rect;  GetClientRect(rect);
	ClientToScreen(&rect);
	CPoint ptPopup(rect.left, rect.bottom);
	m_wndList->Invoke(this, ptPopup, rect.Width(), m_nBitCount, m_bitNames);
}

void CBitFieldCtrl::OnLButtonUp(UINT nFlags, CPoint point) 
{
	int nIndex = -1;

	for(int i = 0; i < m_nBitCount; i++)
	{
		if(m_bitRect[i].PtInRect(point))
		{
			nIndex = i;
			break;
		}
	}

	CButton::OnLButtonUp(nFlags, point);

	if(nIndex == -1) return;
	
	m_bJustGone = FALSE;
	m_dwValue ^= (1 << nIndex);
	InvalidateRect(m_bitRect[nIndex]);

	GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), BFCN_BITSCHANGED), (LPARAM)GetSafeHwnd());
}

///////////////////////////////////////////////////////////
//
//  CBitCtrlList class -- implementation for
//    drop down list controls in the toolbars
//

BEGIN_MESSAGE_MAP(CBitCtrlList, CMiniFrameWnd)
	ON_WM_CREATE()
	ON_WM_ACTIVATE()
	ON_MESSAGE(WM_SETFONT, OnSetFont)
	ON_WM_MEASUREITEM()
	ON_WM_DRAWITEM()
END_MESSAGE_MAP()

BOOL CBitCtrlList::Create(CWnd* pParentWnd)
{
	LPCTSTR className = AfxRegisterWndClass(CS_SAVEBITS,
		AfxGetApp()->LoadStandardCursor(IDC_ARROW), (HBRUSH)(COLOR_WINDOW + 1), NULL);

	DWORD dwWndStyle = WS_POPUP | MFS_SYNCACTIVE;

	BOOL rval = CMiniFrameWnd::CreateEx(WS_EX_TOOLWINDOW, className, NULL, dwWndStyle,
		CRect(0, 0, 0, 0), pParentWnd);

	if(!rval) return FALSE;

	m_wndList.Create(WS_CHILD | WS_VISIBLE | WS_VSCROLL | LBS_NOTIFY | LBS_NOINTEGRALHEIGHT |
		LBS_HASSTRINGS | LBS_OWNERDRAWFIXED | WS_BORDER, CRect(0, 0, 0, 0), this, 0);

	return rval;
}

void CBitCtrlList::Invoke(CBitFieldCtrl* pParent, CPoint pt, int nWidth, int nStrings, CString* aStrings)
{
	m_pParent = pParent;

	m_bValid = FALSE;
	m_bCanceled = FALSE;

	m_wndList.ResetContent();

	for(int i = 0; i < nStrings; i++)
		m_wndList.AddString(aStrings[i]);

	BOOL bScrollbar = FALSE;
	
	int nItemHeight = m_wndList.GetItemHeight(0);
	int nTotalHeight = m_wndList.GetCount() * nItemHeight;
	if(nTotalHeight > nItemHeight * 12)
	{
		nTotalHeight = nItemHeight * 12;
		bScrollbar = TRUE;
	}

	int nTotalWidth = 0;

	for(i = 0; i < m_wndList.GetCount(); i++)
	{
		CString strItem;  m_wndList.GetText(i, strItem);
		CSize extent = m_wndList.GetDC()->GetTextExtent(strItem);
		if(extent.cx > nTotalWidth) nTotalWidth = extent.cx;
	}

	if(nWidth > nTotalWidth) nTotalWidth = nWidth;
	if(bScrollbar) nTotalWidth += GetSystemMetrics(SM_CXVSCROLL);
	
	int cxScreen = GetSystemMetrics(SM_CXSCREEN);
	int cyScreen = GetSystemMetrics(SM_CYSCREEN);

	SetWindowPos(NULL, pt.x, pt.y,
		nTotalWidth, nTotalHeight + 2, SWP_SHOWWINDOW);
	m_wndList.SetWindowPos(NULL, 0, 0, nTotalWidth, nTotalHeight + 2, SWP_SHOWWINDOW);
}

void CBitCtrlList::OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized) 
{
	CMiniFrameWnd::OnActivate(nState, pWndOther, bMinimized);

	if(nState == WA_INACTIVE)
	{
		m_pParent->m_bJustGone = TRUE;
		CancelWindow();
	}
}

void CBitCtrlList::CancelWindow()
{
	ShowWindow(SW_HIDE);
}

LONG CBitCtrlList::OnSetFont(UINT hFont, LONG fRedraw)
{
	m_wndList.SendMessage(WM_SETFONT, hFont, fRedraw);
	return NULL;
}

BOOL CBitCtrlList::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	if(HIWORD(wParam) == LBN_SELCHANGE)
	{
		int nIndex = m_wndList.GetCurSel();
		m_pParent->m_dwValue ^= (1 << nIndex);
		m_pParent->InvalidateRect(m_pParent->m_bitRect[nIndex]);
		
		CRect itemRect;
		m_wndList.GetItemRect(nIndex, itemRect);
		m_wndList.InvalidateRect(itemRect);

		m_pParent->GetOwner()->SendMessage(WM_COMMAND,
			MAKELONG(m_pParent->GetDlgCtrlID(), BFCN_BITSCHANGED), (LPARAM)m_pParent->GetSafeHwnd());
	}

	return CMiniFrameWnd::OnCommand(wParam, lParam);
}

void CBitCtrlList::OnMeasureItem(int nID, LPMEASUREITEMSTRUCT lpmis)
{
	lpmis->itemHeight = 15;
}

void CBitCtrlList::OnDrawItem(int nID, LPDRAWITEMSTRUCT lpdis)
{
	// Only one control will ever need painting, and since its ID changes,
	// it's counterproductive to confirm this.

	CDC dc;
	dc.Attach(lpdis->hDC);

	CRect rect = lpdis->rcItem;
	UINT nIndex = lpdis->itemID;

	if(nIndex != (UINT)-1)
	{
		CString strText;  m_wndList.GetText(nIndex, strText);

		CRect textRect = rect;
		textRect.left += 2;

		if(m_pParent->m_images.GetSafeHandle())
		{
			m_pParent->m_images.Draw(&dc, ((m_pParent->m_dwValue) >> nIndex) & 0x1,
				CPoint(textRect.left, textRect.top + (rect.Height() - 13) / 2), ILD_NORMAL);
		}

		textRect.left += 16;

		dc.FillSolidRect(textRect, ::GetSysColor((lpdis->itemState & ODS_SELECTED) ?
			COLOR_HIGHLIGHT : COLOR_WINDOW));
		if(lpdis->itemState & ODS_SELECTED) dc.DrawFocusRect(textRect);

		textRect.left += 2;

		CFont* oldFont = dc.SelectObject(m_wndList.GetFont());
		dc.SetBkMode(TRANSPARENT);
		dc.SetTextColor(::GetSysColor((lpdis->itemState & ODS_SELECTED) ?
			COLOR_HIGHLIGHTTEXT : COLOR_WINDOWTEXT));
		dc.DrawText(strText, &textRect, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
		dc.SelectObject(oldFont);
	}

	dc.Detach();
}

///////////////////////////////////////////////////////////
//
//  XBitCtrlList class -- implementation for
//    drop down list controls in the toolbars
//

BEGIN_MESSAGE_MAP(XBitCtrlList, CListBox)
	ON_WM_MOUSEMOVE()
END_MESSAGE_MAP()

void XBitCtrlList::OnMouseMove(UINT nFlags, CPoint point) 
{
	CListBox::OnMouseMove(nFlags, point);

	BOOL bOutside;
	int nSel = (int)ItemFromPoint(point, bOutside);
	if(bOutside) nSel = -1;

	if(GetCurSel() != nSel) SetCurSel(nSel);
}

