#ifndef __FF4DATA_H__
#define __FF4DATA_H__

#include "FF4Edit.h"

#define RGB555(r, g, b) (WORD)((BYTE)(b) | ((BYTE)(g) << 5) | ((BYTE)(r) << 10))
#define R555(clr) ((clr >> 10) & 0x1F)
#define G555(clr) ((clr >> 5) & 0x1F)
#define B555(clr) (clr & 0x1F)

DWORD DDColor(BYTE r, BYTE g, BYTE b);
BYTE DDGetR(DWORD color);
BYTE DDGetG(DWORD color);
BYTE DDGetB(DWORD color);
DWORD Convert555ToDDColor(WORD color);
DWORD SubtractDDColor(DWORD src, DWORD dest, BYTE opacity);

#define SendCommandMessage(wnd, msg) SendMessage(WM_COMMAND, MAKELONG((wnd)->GetDlgCtrlID(), (msg)), (LPARAM)(wnd)->GetSafeHwnd())

DWORD SMCToLoROM(DWORD dwSMC);
DWORD LoROMToSMC(DWORD dwLoROM);
CString FixedTextToCString(BYTE* pText, int nLength, DWORD tag, DWORD value, ... );

#define TTCS_BASE					(0x81000000)
#define TTCS_TrimTrailingSpaces		(TTCS_BASE + 1)

///////////////////////////////////////////////////////////

class CFF4Doc;
class WorldData;

enum ObjectTypes
{
	ObjWorldData = 1,
	ObjLocationData
};

class FF4Object
{
public:
	int objectType;
};

class EventCommand
{
public:
	BYTE command;
	BYTE paramCount;
	BYTE params[8];
};

class Event
{
public:
	Event();
	~Event();

	WORD size;
	CTypedPtrArray<CPtrArray, EventCommand*> code;
};

class EnemyCombination
{
public:
	BYTE flags1;
	BYTE enemy1;
	BYTE enemy2;
	BYTE enemy3;
	BYTE flags2;
	BYTE arrangement;
	BYTE flags3;
	BYTE unknown;
};

#define ECF1_UNKNOWN1			0x07
#define ECF1_FORCEBACKATTACK	0x08
#define ECF1_BOSSDEATH			0x10
#define ECF1_UNKNOWN2			0x60
#define ECF1_EGG				0x80

#define ECF3_CANTRUN			0x01
#define ECF3_NOTFATAL			0x02
#define ECF3_MUSIC				0x0C
#define ECF3_HUMANSTYLE			0x10
#define ECF3_AUTOBATTLE			0x20
#define ECF3_FLOAT				0x40
#define ECF3_TRANSPARENT		0x80

class Enemy
{
public:
	Enemy();
	~Enemy();

	CFF4Doc* pDoc;

	BYTE name[8];
	CSize size;
	WORD gold;
	WORD experience;
	WORD hp;

	BYTE* tiles;
};

class Item
{
public:
	Item();
	~Item();

	BYTE name[9];
};

class Character
{
public:
	Character();
	~Character();

	BYTE name[6];
};

class LocationTilePart
{
public:
	BYTE tilePart;
	BYTE palette;
};

class Entrance
{
public:
	BYTE srcX;
	BYTE srcY;
	BYTE destination;
	BYTE destX;
	BYTE destY;
};

class Trigger
{
public:
	BYTE srcX;
	BYTE srcY;
	DWORD offset;
};

class GlobalLocationData : public FF4Object
{
public:
	GlobalLocationData();
	~GlobalLocationData();

	CFile* pFile;

	BYTE* tiles[16];
	BYTE* maps[256];
	CSize mapSize;
	WORD palettes[32][64];
	LocationTilePart tileFormations[16][512];
	WORD tileProperties[16][128];
};

class LocationData : public FF4Object
{
public:
	LocationData();
	~LocationData();

	CFF4Doc* pDoc;
	CFile* pFile;
	GlobalLocationData* gLocationData;
	CString name;

	BYTE battleBackground;
	BYTE mapIndex;
	BYTE tileSet;
	BYTE NPCIndex;
	BYTE borderTile;
	BYTE paletteIndex;
	BYTE NPCPaletteIndex;
	BYTE musicIndex;
	BYTE backgroundTile;
	BYTE scrollSpecs;
	BYTE UK2;
	BYTE titleIndex;
	BYTE UK3;

	CTypedPtrArray<CPtrArray, Entrance*> entrances;

	Entrance* FindEntranceFromSource(BYTE x, BYTE y);

	BYTE GetMapTile(int x, int y) { return gLocationData->maps[mapIndex][y * gLocationData->mapSize.cx + x]; }
	void SetMapTile(int x, int y, BYTE bTile) { gLocationData->maps[mapIndex][y * gLocationData->mapSize.cx + x] = bTile; }

	// Program persistent data
	CPoint ppdLastScrollPos;
};

#define LD_BATTLEBACKGROUND		0x0F
#define LD_BATTLEALTPALETTE		0x40
#define LD_BATTLEMAGNETIC		0x80

#define SSF_TRANSPARENT			0x01
#define SSF_VERTSTICKY			0x02
#define SSF_HORZSTICKY			0x04
#define SSF_DIRECTION			0x30
#define SSF_SPEED				0xC0

#define DIRECTION_UP			0x00
#define DIRECTION_RIGHT			0x01
#define DIRECTION_DOWN			0x02
#define DIRECTION_LEFT			0x03

#define DIRECTION_BACKWARD		DIRECTION_UP
#define DIRECTION_FORWARD		DIRECTION_DOWN

#define LDF_WALK				0x0001
#define LDF_UNKNOWN1			0x0006
#define LDF_SAVEPOINT			0x0008
#define LDF_UNKNOWN2			0x00F0

#define LDF_WALKDAMAGE			0x0100
#define LDF_UNKNOWN3			0x0200
#define LDF_HIDECHARACTER		0x0400
#define LDF_TOPHALFCHARACTER	0x0800
#define LDF_EXIT				0x1000
#define LDF_UNKNOWN4			0x6000
#define LDF_ENTRANCE			0x8000

#define MAP_OVERWORLD 0
#define MAP_UNDERGROUND 1
#define MAP_MOON 2

class WorldData : public FF4Object
{
public:
	WorldData();
	~WorldData();

public:
	CFF4Doc* pDoc;

	CString name;
	CSize size;
	BYTE* map;
	BYTE* tiles;
	BYTE tileFormations[512];
	WORD tileProperties[128];
	BYTE tilePalettes[256];
	WORD palette[64];
	BYTE enemyLayout[64];
	BYTE enemyOdds[64];

	CTypedPtrArray<CPtrArray, Entrance*> entrances;

	BOOL ReadMapData(int nMap);
	BOOL ReadTileData(DWORD dwOffset);
	BOOL ReadTileFormations(DWORD dwOffset);
	BOOL ReadTileProperties(DWORD dwOffset);
	BOOL ReadTilePalettes(DWORD dwOffset);
	BOOL ReadPalettes(DWORD dwOffset);
	BOOL ReadEntrances(DWORD dwOffset);
	BOOL ReadEnemyLayout(DWORD dwOffset, DWORD dwLength);
	BOOL ReadEnemyOdds(DWORD dwOffset, DWORD dwLength);

	BOOL StoreMapData(int nMap);
	BOOL StoreTileData(DWORD dwOffset);
	BOOL StoreTileFormations(DWORD dwOffset);
	BOOL StoreTileProperties(DWORD dwOffset);
	BOOL StoreTilePalettes(DWORD dwOffset);
	BOOL StorePalettes(DWORD dwOffset);
	BOOL StoreEntrances(DWORD dwOffset, BOOL bExpand);
	BOOL StoreEnemyLayout(DWORD dwOffset, DWORD dwLength);
	BOOL StoreEnemyOdds(DWORD dwOffset, DWORD dwLength);

	Entrance* FindEntranceFromSource(BYTE x, BYTE y);

	BYTE GetMapTile(int x, int y) { return map[y * size.cx + x]; }
	void SetMapTile(int x, int y, BYTE bTile) { map[y * size.cx + x] = bTile; }

	// Program persistent data
	CPoint ppdLastScrollPos;
};

#define WDF_CHARAWALK			0x0001
#define WDF_CHOCOWALK			0x0002
#define WDF_BLACKCHOCOFLY		0x0004
#define WDF_BLACKCHOCOLAND		0x0008
#define WDF_HOVERCRAFTMOVE		0x0010
#define WDF_AIRSHIPFLY			0x0020
#define WDF_CHARAWALK2			0x0040
#define WDF_BIGWHALEFLY			0x0080

#define WDF_BATTLEBACKGROUND	0x0700
#define WDF_SHOWCHARATOPHALF	0x0800
#define WDF_AIRSHIPLAND			0x1000
#define WDF_UNKNOWN2			0x2000
#define WDF_ENEMYENCOUNTERS		0x4000
#define WDF_ENTRANCE			0x8000

// Tags
#define TAG_END					0x00000000

#endif