// FF4Doc.cpp : implementation of the CFF4Doc class
//

#include "stdafx.h"
#include "FF4Edit.h"

#include "FF4Doc.h"

#include "MainFrame.h"
#include "EditorInfoView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFF4Doc

IMPLEMENT_DYNCREATE(CFF4Doc, CDocument)

BEGIN_MESSAGE_MAP(CFF4Doc, CDocument)
	//{{AFX_MSG_MAP(CFF4Doc)
//	ON_COMMAND(ID_FILE_RUN, OnFileRun)
	ON_COMMAND(ID_VIEW_PROPERTIES, OnViewTileDialog)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PROPERTIES, OnUpdateViewTileDialog)
	ON_COMMAND(ID_VIEW_TILEWINDOW, OnViewTileWindow)
	ON_UPDATE_COMMAND_UI(ID_VIEW_TILEWINDOW, OnUpdateViewTileWindow)
	ON_COMMAND(ID_VIEW_TRIGGERS, OnViewTriggerWindow)
	ON_UPDATE_COMMAND_UI(ID_VIEW_TRIGGERS, OnUpdateViewTriggerWindow)
	ON_COMMAND(ID_VIEW_GENERICPROP, OnViewGenericPropWindow)
	ON_UPDATE_COMMAND_UI(ID_VIEW_GENERICPROP, OnUpdateViewGenericPropWindow)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFF4Doc construction/destruction

CFF4Doc::CFF4Doc()
{
	m_bValid = FALSE;

	worldData = NULL;
	events = NULL;
	gLocationData = NULL;
}

CFF4Doc::~CFF4Doc()
{
}

BOOL CFF4Doc::OnNewDocument()
{
	if(!CDocument::OnNewDocument())
		return FALSE;

	m_bValid = FALSE;
	OnProperties();

	return TRUE;
}

void CFF4Doc::DeleteContents() 
{
	m_bValid = FALSE;

	for(int i = 0; i < locations.GetSize(); i++)
		delete locations[i];
	locations.RemoveAll();

	if(gLocationData) { delete gLocationData; gLocationData = NULL; }
	if(events) { delete [] events;  events = NULL; }
	if(worldData) { delete [] worldData; worldData = NULL; }

	CDocument::DeleteContents();

	if(AfxGetMainWnd())
	{
		FLEXPANE fp;
		fp.nID = 0;
		fp.pClass = RUNTIME_CLASS(CEditorInfoView);
		fp.dwData = 0;
		AfxGetMainWnd()->SendMessage(WM_UPDATEFLEXPANE, (WPARAM)&fp);
		UpdateAllViews((CView*)((CMainFrame*)AfxGetMainWnd())->m_wndSplitter.GetPane(0, 1));
	}
}

/////////////////////////////////////////////////////////////////////////////
// CFF4Doc serialization

int g_eventParamsD0[] = {

//          0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F
//
/* D */		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1,
/* E */		1, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1,
/* F */		1, 1, 1, 1, 1, 1, 1, 1,	1, 1, 1, 1, 0, 1, 4, 0
};

BOOL CFF4Doc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if(IsModified())
		TRACE0("Warning: OnOpenDocument replaces an unsaved document.\n");

	CFileException fe;
	m_pFile = GetFile(lpszPathName,
		CFile::modeRead|CFile::shareDenyWrite, &fe);
	if (m_pFile == NULL)
	{
		ReportSaveLoadException(lpszPathName, &fe,
			FALSE, AFX_IDP_FAILED_TO_OPEN_DOC);

		m_bValid = FALSE;
		return FALSE;
	}

	DeleteContents();
	SetModifiedFlag();  // dirty during de-serialize

	CWaitCursor wait;
	if(m_pFile->GetLength() == 0)
	{
		ReportSaveLoadException(lpszPathName, &fe,
			FALSE, AFX_IDP_FAILED_TO_OPEN_DOC);

		m_bValid = FALSE;
		return FALSE;
	}

	// Check to see if the file is valid (ie, the American FF3 ROM and version 1.0).
	char szV10Title[22] = "FINAL FANTASY II     ";
	char szROMTitle[22];
	m_pFile->Seek(0x81C0, CFile::begin);
	m_pFile->Read(szROMTitle, 21);
	
	if(memcmp(szV10Title, szROMTitle, 21) != 0)
	{
		AfxMessageBox("The ROM you are attempting to load\nis not the American FF2 ROM.");
		m_bValid = FALSE;
		return FALSE;
	}

	BYTE bVersion = 0;
	m_pFile->Seek(0x81DB, CFile::begin);
	m_pFile->Read(&bVersion, 1);
	
	if(bVersion != 0)
	{
		AfxMessageBox("The ROM you are attempting to load\nis not version 1.0 of the American\nFF2 ROM.  Let me know if you see this, because\nI was not aware of other versions of\nFF3 being available.");
		m_bValid = FALSE;
		return FALSE;
	}
	
	// Load the actual file.
	worldData = new WorldData[3];

	worldData[MAP_OVERWORLD].pDoc = this;
	worldData[MAP_OVERWORLD].name = "Overworld";
	worldData[MAP_OVERWORLD].ReadMapData(MAP_OVERWORLD);
	worldData[MAP_OVERWORLD].ReadTileData(0xE8200);
	worldData[MAP_OVERWORLD].ReadTileFormations(0xA0200);
	worldData[MAP_OVERWORLD].ReadTileProperties(0xA0C80);
	worldData[MAP_OVERWORLD].ReadTilePalettes(0xA0800);
	worldData[MAP_OVERWORLD].ReadPalettes(0xA0B00);
	worldData[MAP_OVERWORLD].ReadEntrances(0xD0060);
	worldData[MAP_OVERWORLD].ReadEnemyLayout(0x74742, 64);
	worldData[MAP_OVERWORLD].ReadEnemyOdds(0x74500, 64);
//	worldData[MAP_OVERWORLD].ReadLocations();

	worldData[MAP_UNDERGROUND].pDoc = this;
	worldData[MAP_UNDERGROUND].name = "Underground";
	worldData[MAP_UNDERGROUND].ReadMapData(MAP_UNDERGROUND);
	worldData[MAP_UNDERGROUND].ReadTileData(0xEA200);
	worldData[MAP_UNDERGROUND].ReadTileFormations(0xA0400);
	worldData[MAP_UNDERGROUND].ReadTileProperties(0xA0D80);
	worldData[MAP_UNDERGROUND].ReadTilePalettes(0xA0900);
	worldData[MAP_UNDERGROUND].ReadPalettes(0xA0B80);
	worldData[MAP_UNDERGROUND].ReadEntrances(0xD0062);
//	worldData[MAP_UNDERGROUND].ReadLocations();

	worldData[MAP_MOON].pDoc = this;
	worldData[MAP_MOON].name = "The Moon";
	worldData[MAP_MOON].ReadMapData(MAP_MOON);
	worldData[MAP_MOON].ReadTileData(0xEC200);
	worldData[MAP_MOON].ReadTileFormations(0xA0600);
	worldData[MAP_MOON].ReadTileProperties(0xA0E80);
	worldData[MAP_MOON].ReadTilePalettes(0xA0A00);
	worldData[MAP_MOON].ReadPalettes(0xA0C00);
	worldData[MAP_MOON].ReadEntrances(0xD0064);
//	worldData[MAP_MOON].ReadLocations();

	gLocationData = new GlobalLocationData;

	ReadLocations();
	
	int i = 0;

	m_pFile->Seek(0x57200, CFile::begin);
	m_pFile->Read(font, 0x1000);

	m_pFile->Seek(0x689D0, CFile::begin);
	m_pFile->Read(fontPalette, 16);

	m_pFile->Seek(0x99900, CFile::begin);
	m_pFile->Read(&dteTable, 256);

	m_pFile->Seek(0x78200, CFile::begin);
	for(i = 0; i < 256; i++)
		m_pFile->Read(items[i].name, 9);

	for(i = 0; i < 16; i++)
	{
		BYTE bAppr = 0;
		m_pFile->Seek(0x7AB00 + (i * 0x20), CFile::begin);
		m_pFile->Read(&bAppr, 1);
		bAppr &= 0x3F;  bAppr--;

		m_pFile->Seek(0x7A910 + (bAppr * 6), CFile::begin);
		m_pFile->Read(characters[i + 1].name, 6);
	}

	m_pFile->Seek(0x74996, CFile::begin);
	for(i = 0; i < 11; i++)
		m_pFile->Read(enemySets[i], 8);

	m_pFile->Seek(0x70200, CFile::begin);
	for(i = 0; i < 512; i++)
		m_pFile->Read(&enemyCombos[i], 8);

	m_pFile->Seek(0x7015E, CFile::begin);
	for(i = 0; i < 256; i++)
	{
		BYTE size[2];
		m_pFile->Read(&size, 2);
		enemySize[i].cx = size[0];
		enemySize[i].cy = size[1];
	}

	m_pFile->Seek(0x72200, CFile::begin);
	for(i = 0; i < 256; i++)
	{
		m_pFile->Read(&enemies[i].gold, 2);
	}

	m_pFile->Seek(0x723C0, CFile::begin);
	for(i = 0; i < 256; i++)
	{
		m_pFile->Read(&enemies[i].experience, 2);
	}

	m_pFile->Seek(0x71A00, CFile::begin);
	for(i = 0; i < 256; i++)
	{
		m_pFile->Read(&enemies[i].name, 8);
	}

	for(i = 0; i < 256; i++)
	{
		m_pFile->Seek(0x72A60 + (i * 0x0E), CFile::begin);
		BYTE bDummy;
		m_pFile->Read(&bDummy, 1);
		m_pFile->Read(&enemies[i].hp, 2);
	}

	for(i = 0; i < 256; i++)
	{
		BYTE bSize, bPalette;
		WORD wPtr;

		m_pFile->Seek(0x7CC00 + (i * 4), CFile::begin);
		m_pFile->Read(&bSize, 1);
		m_pFile->Read(&bPalette, 1);
		m_pFile->Read(&wPtr, 2);

		enemies[i].size.cx = enemySize[bSize].cx;
		enemies[i].size.cy = enemySize[bSize].cy;
//		enemies[i].palette = bPalette;

		m_pFile->Seek(0x47200 + wPtr, CFile::begin);
		enemies[i].tiles = new BYTE[enemySize[bSize].cx * enemySize[bSize].cy * 3];
		m_pFile->Read(enemies[i].tiles, enemySize[bSize].cx * enemySize[bSize].cy * 3);
	}

	events = new Event[256];
	BYTE* tempEvent = new BYTE[16384];		// Should be enough to hold any event.

	WORD wEvents[256];
	m_pFile->Seek(0x90200, CFile::begin);
	m_pFile->Read(&wEvents, 512);

	for(i = 0; i < 256; i++)
	{
		BOOL bEnd = FALSE;
		int nPos = 0;
		m_pFile->Seek(0x90400 + wEvents[i], CFile::begin);

		while(!bEnd)
		{
			m_pFile->Read(&tempEvent[nPos], 1);
			if(tempEvent[nPos] == 0xFF) bEnd = TRUE;
			nPos++;
		}

		for(int j = 0; j < nPos; j++)
		{
			EventCommand* pCommand = new EventCommand;

			pCommand->command = tempEvent[j];
			ZeroMemory(pCommand->params, 4);

			if(pCommand->command >= 0xD0)
			{
				int nParams = g_eventParamsD0[pCommand->command - 0xD0];
				pCommand->paramCount = nParams;

				CopyMemory(pCommand->params, tempEvent + (++j), nParams);
				j += nParams - 1;
			}

			events[i].code.Add(pCommand);
		}
	}

	if(tempEvent) delete tempEvent;

	m_bValid = TRUE;

	ReleaseFile(m_pFile, FALSE);
	SetModifiedFlag(FALSE);     // start off with unmodified

	return TRUE;
}

BOOL CFF4Doc::OnSaveDocument(LPCTSTR lpszPathName) 
{
	CFileException fe;
	m_pFile = NULL;
	m_pFile = GetFile(lpszPathName, CFile::modeReadWrite | CFile::shareExclusive, &fe);

	if(m_pFile == NULL)
	{
		ReportSaveLoadException(lpszPathName, &fe,
			TRUE, AFX_IDP_INVALID_FILENAME);
		return FALSE;
	}

	CWaitCursor wait;

	// Set the expansion offset to the end of the ROM.  Various save routines
	// will use this if they need to expand, then add to it once they've done
	// so.  This makes the optimal use of space by not fixing data at static
	// locations in the ROM.
	m_dwExpansionOffset = 0x100200;

	// Check to see if there are more triggers than can fit in
	// the space allowed.  It is of no importance if one map has
	// more triggers -as long as- another has had enough removed
	// that they will still fit.  We check for -total- number of
	// triggers here.
	BOOL bExpandEntr = FALSE;
	if(worldData[MAP_OVERWORLD].entrances.GetSize() + worldData[MAP_UNDERGROUND].entrances.GetSize() +
		worldData[MAP_MOON].entrances.GetSize() > 82) bExpandEntr = TRUE;

	// Data requiring expansion.
	worldData[MAP_OVERWORLD].StoreMapData(MAP_OVERWORLD);
	worldData[MAP_UNDERGROUND].StoreMapData(MAP_UNDERGROUND);
	worldData[MAP_MOON].StoreMapData(MAP_MOON);

	worldData[MAP_OVERWORLD].StoreEntrances(0xD0060, bExpandEntr);
	worldData[MAP_UNDERGROUND].StoreEntrances(0xD0062, bExpandEntr);
	worldData[MAP_MOON].StoreEntrances(0xD0064, bExpandEntr);

	// Data not requiring expansion.
	worldData[MAP_OVERWORLD].StoreTileData(0xE8200);
	worldData[MAP_UNDERGROUND].StoreTileData(0xEA200);
	worldData[MAP_MOON].StoreTileData(0xEC200);

	worldData[MAP_OVERWORLD].StoreTileFormations(0xA0200);
	worldData[MAP_UNDERGROUND].StoreTileFormations(0xA0400);
	worldData[MAP_MOON].StoreTileFormations(0xA0600);

	worldData[MAP_OVERWORLD].StoreTileProperties(0xA0C80);
	worldData[MAP_UNDERGROUND].StoreTileProperties(0xA0D80);
	worldData[MAP_MOON].StoreTileProperties(0xA0E80);

	worldData[MAP_OVERWORLD].StoreTilePalettes(0xA0800);
	worldData[MAP_UNDERGROUND].StoreTilePalettes(0xA0900);
	worldData[MAP_MOON].StoreTilePalettes(0xA0A00);

	worldData[MAP_OVERWORLD].StorePalettes(0xA0B00);
	worldData[MAP_UNDERGROUND].StorePalettes(0xA0B80);
	worldData[MAP_MOON].StorePalettes(0xA0C00);

	worldData[MAP_OVERWORLD].StoreEnemyLayout(0x74742, 64);

	worldData[MAP_OVERWORLD].StoreEnemyOdds(0x74500, 64);

//	StoreLocations();

	// It's possible that the file was previously expanded, but
	// changes in the data caused expansion to be unnecessary.
	// We forcefully set the end of the file to truncate any
	// previously expanded data that isn't being used.
	m_pFile->Seek(m_dwExpansionOffset, CFile::begin);
	::SetEndOfFile((HANDLE)m_pFile->m_hFile);
	ReleaseFile(m_pFile, FALSE);

	SetModifiedFlag(FALSE);     // back to unmodified

	return TRUE;        // success
}

BOOL CFF4Doc::ReadLocations()
{
	gLocationData->mapSize = CSize(32, 32);

	m_pFile->Seek(0xA1000, CFile::begin);
	m_pFile->Read(gLocationData->tileProperties, 0x1000);

	m_pFile->Seek(0xA2000, CFile::begin);
	m_pFile->Read(gLocationData->tileFormations, 0x4000);

	m_pFile->Seek(0xA6000, CFile::begin);
	m_pFile->Read(gLocationData->palettes, 0x1000);

	for(int t = 0; t < 16; t++)
	{
		gLocationData->tiles[t] = new BYTE[0x1800];

		WORD wTileOffset = 0;
		m_pFile->Seek(0xF0200 + t * 2, CFile::begin);
		m_pFile->Read(&wTileOffset, 2);

		BYTE bBank = 0;
		m_pFile->Seek(0xAB304 + t, CFile::begin);
		m_pFile->Read(&bBank, 1);

		if(bBank == 0x15)
			m_pFile->Seek(0xA0200 + wTileOffset, CFile::begin);
		else if(bBank == 0x1D)
			m_pFile->Seek(0xE0200 + wTileOffset, CFile::begin);
		else if(bBank == 0x1E)
			m_pFile->Seek(0xE8200 + wTileOffset, CFile::begin);
		else if(bBank == 0x1F)
			m_pFile->Seek(0xF0200 + wTileOffset, CFile::begin);

		m_pFile->Read(gLocationData->tiles[t], 0x1800);
	}

	// Load the maps.
	WORD wMaps[256];
	m_pFile->Seek(0xB8200, CFile::begin);
	m_pFile->Read(&wMaps, 512);

	for(int i = 0; i < 256; i++)
	{
		gLocationData->maps[i] = new BYTE[32 * 32];

		m_pFile->Seek(0xB8200 + wMaps[i], CFile::begin);
		for(int t = 0; t < 32 * 32; t++)
		{
			int xPos = 0;

			BYTE bTile = 0;
			m_pFile->Read(&bTile, 1);

			if(bTile >= 0x80)
			{
				BYTE nCount = 0;
				m_pFile->Read(&nCount, 1);
				nCount++;
				if(nCount == 0) nCount = 255;

				for(int j = 0; j < nCount; j++, t++)
					gLocationData->maps[i][t] = bTile - 0x80;

				t--;
			}
			else
			{
				gLocationData->maps[i][t] = bTile;
			}
		}
	}

	for(int n = 0; n < 0xD0; n++)
	{
		LocationData* pLoc = new LocationData;
		locations.Add(pLoc);

		locations[n]->pDoc = this;
		locations[n]->pFile = m_pFile;
		locations[n]->gLocationData = gLocationData;
		
		// Load the town properties.
		m_pFile->Seek(0xA9E84 + (n * 0xD), CFile::begin);
		m_pFile->Read(&locations[n]->battleBackground, 1);
		m_pFile->Read(&locations[n]->mapIndex, 1);
		m_pFile->Read(&locations[n]->tileSet, 1);
		m_pFile->Read(&locations[n]->NPCIndex, 1);
		m_pFile->Read(&locations[n]->borderTile, 1);
		m_pFile->Read(&locations[n]->paletteIndex, 1);
		m_pFile->Read(&locations[n]->NPCPaletteIndex, 1);
		m_pFile->Read(&locations[n]->musicIndex, 1);
		m_pFile->Read(&locations[n]->backgroundTile, 1);
		m_pFile->Read(&locations[n]->scrollSpecs, 1);
		m_pFile->Read(&locations[n]->UK2, 1);
		m_pFile->Read(&locations[n]->titleIndex, 1);
		m_pFile->Read(&locations[n]->UK3, 1);

		int i = locations[n]->mapIndex;

		CString s; s.Format("Location 0x%02X (%x)", n, locations[n]->tileSet);
		locations[n]->name = s;

/*		DWORD dwActualOffset, dwLength;
		WORD wEntranceOffset, wNextOffset;

		m_pFile->Seek(0xA8200 + n * 2, CFile::begin);
		m_pFile->Read(&wEntranceOffset, 2);
		dwActualOffset = 0xA8500 + wEntranceOffset;

		m_pFile->Seek(0xA8200 + (n + 1) * 2, CFile::begin);
		m_pFile->Read(&wNextOffset, 2);

		dwLength = wNextOffset - wEntranceOffset;

/*		m_pFile->Seek(dwActualOffset, CFile::begin);
		for(UINT e = 0; e < dwLength / 5; e++)
		{
			locations[n].entrances[e] = new Entrance;
			m_pFile->Read(locations[n].entrances[e], 5);
		}*/
	}

	return TRUE;
}

BOOL CFF4Doc::StoreLocations()
{
/*	WORD wMaps[256];
	m_pFile->Seek(0xB8200, CFile::begin);
	m_pFile->Read(&wMaps, 512);

	locations.SetSize(0xD0);

	m_pFile->Seek(0xA1000, CFile::begin);
	m_pFile->Read(gLocationData->tileProperties, 0x1000);

	m_pFile->Seek(0xA2000, CFile::begin);
	m_pFile->Read(gLocationData->tileFormations, 0x4000);

	m_pFile->Seek(0xA6000, CFile::begin);
	m_pFile->Read(gLocationData->palettes, 0x1000);

	for(int t = 0; t < 16; t++)
	{
		gLocationData->tiles[t] = new BYTE[0x1800];

		WORD wTileOffset = 0;
		m_pFile->Seek(0xF0200 + t * 2, CFile::begin);
		m_pFile->Read(&wTileOffset, 2);

		BYTE bBank = 0;
		m_pFile->Seek(0xAB304 + t, CFile::begin);
		m_pFile->Read(&bBank, 1);

		if(bBank == 0x15)
			m_pFile->Seek(0xA0200 + wTileOffset, CFile::begin);
		else if(bBank == 0x1D)
			m_pFile->Seek(0xE0200 + wTileOffset, CFile::begin);
		else if(bBank == 0x1E)
			m_pFile->Seek(0xE8200 + wTileOffset, CFile::begin);
		else if(bBank == 0x1F)
			m_pFile->Seek(0xF0200 + wTileOffset, CFile::begin);

		m_pFile->Read(gLocationData->tiles[t], 0x1800);
	}
*/
	for(int n = 0; n < 0xD0; n++)
	{
		// Save the town properties.
		m_pFile->Seek(0xA9E84 + (n * 0xD), CFile::begin);
		m_pFile->Write(&locations[n]->battleBackground, 1);
		m_pFile->Write(&locations[n]->mapIndex, 1);
		m_pFile->Write(&locations[n]->tileSet, 1);
		m_pFile->Write(&locations[n]->NPCIndex, 1);
		m_pFile->Write(&locations[n]->borderTile, 1);
		m_pFile->Write(&locations[n]->paletteIndex, 1);
		m_pFile->Write(&locations[n]->NPCPaletteIndex, 1);
		m_pFile->Write(&locations[n]->musicIndex, 1);
		m_pFile->Write(&locations[n]->backgroundTile, 1);
		m_pFile->Write(&locations[n]->scrollSpecs, 1);
		m_pFile->Write(&locations[n]->UK2, 1);
		m_pFile->Write(&locations[n]->titleIndex, 1);
		m_pFile->Write(&locations[n]->UK3, 1);

/*		int i = locations[n]->mapIndex;

		CString s; s.Format("Location 0x%02X (%x)", n, locations[n]->tileSet);
		locations[n]->name = s;

		m_pFile->Seek(0xB8200 + wMaps[i], CFile::begin);
		for(int t = 0; t < 32 * 32; t++)
		{
			int xPos = 0;

			BYTE bTile = 0;
			m_pFile->Read(&bTile, 1);

			if(bTile >= 0x80)
			{
				BYTE nCount = 0;
				m_pFile->Read(&nCount, 1);
				nCount++;
				if(nCount == 0) nCount = 255;

				for(int j = 0; j < nCount; j++, t++)
					locations[n]->map[t] = bTile - 0x80;

				if(nCount == 255) locations[n]->map[t] = bTile - 0x80;

				t--;
			}
			else
			{
				locations[n]->map[t] = bTile;
			}
		}
*/
/*		DWORD dwActualOffset, dwLength;
		WORD wEntranceOffset, wNextOffset;

		m_pFile->Seek(0xA8200 + n * 2, CFile::begin);
		m_pFile->Read(&wEntranceOffset, 2);
		dwActualOffset = 0xA8500 + wEntranceOffset;

		m_pFile->Seek(0xA8200 + (n + 1) * 2, CFile::begin);
		m_pFile->Read(&wNextOffset, 2);

		dwLength = wNextOffset - wEntranceOffset;

/*		m_pFile->Seek(dwActualOffset, CFile::begin);
		for(UINT e = 0; e < dwLength / 5; e++)
		{
			locations[n].entrances[e] = new Entrance;
			m_pFile->Read(locations[n].entrances[e], 5);
		}*/
	}

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CFF4Doc diagnostics

#ifdef _DEBUG
void CFF4Doc::AssertValid() const
{
	CDocument::AssertValid();
}

void CFF4Doc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CFF4Doc commands

void CFF4Doc::OnProperties()
{
}

void CFF4Doc::OnFileRun() 
{
	CString strPath = "\"" + GetPathName() + "\"";

//	ShellExecute(AfxGetMainWnd()->GetSafeHwnd(), _T("open"), _T("C:\\Games\\zsnes800\\zsnes.exe"),
//		strPath, _T("C:\\Games\\zsnes800"), SW_SHOWNORMAL);
}


void CFF4Doc::DestroyDialogs(CRuntimeClass* pClass)
{
	if(lstrcmp(pClass->m_lpszClassName, _T("CMapView")) == 0)
	{
		m_dlgTiles.DestroyWindow();
		m_wndTiles.DestroyWindow();
		m_wndTriggers.DestroyWindow();
	}
	else if(lstrcmp(pClass->m_lpszClassName, _T("CLocationView")) == 0)
	{
//		m_dlgTiles.DestroyWindow();
		m_wndTiles.DestroyWindow();
		m_wndTriggers.DestroyWindow();
		m_wndGenericProp.DestroyWindow();
	}

	m_szDialogClass.Empty();
}

void CFF4Doc::CreateDialogs(CRuntimeClass* pClass, CView* pView)
{
	if(lstrcmp(pClass->m_lpszClassName, _T("CMapView")) == 0)
	{
		m_dlgTiles.Create(pView);
		m_wndTiles.Create(pView);
		m_wndTriggers.Create(pView);
	}
	else if(lstrcmp(pClass->m_lpszClassName, _T("CLocationView")) == 0)
	{
//		m_dlgTiles.Create(pView);
		m_wndTiles.Create(pView);
		m_wndTriggers.Create(pView);
		m_wndGenericProp.Create(pView);
	}

	m_szDialogClass = pClass->m_lpszClassName;
}

void CFF4Doc::UpdateDialogs(CRuntimeClass* pClass, CView* pView)
{
	if(lstrcmp(pClass->m_lpszClassName, _T("CMapView")) == 0)
	{
		m_dlgTiles.m_pView = pView;
		m_wndTiles.m_pView = pView;
		m_wndTriggers.m_pView = pView;
	}
	else if(lstrcmp(pClass->m_lpszClassName, _T("CLocationView")) == 0)
	{
//		m_dlgTiles.m_pView = pView;
		m_wndTiles.m_pView = pView;
		m_wndTriggers.m_pView = pView;
		m_wndGenericProp.m_pView = pView;
	}

	m_szDialogClass = pClass->m_lpszClassName;
}

void CFF4Doc::OnViewTileDialog() 
{
	if(!m_dlgTiles.IsWindowVisible())
		m_dlgTiles.ShowWindow(SW_SHOW);
	else
		m_dlgTiles.ShowWindow(SW_HIDE);
}

void CFF4Doc::OnUpdateViewTileDialog(CCmdUI* pCmdUI) 
{
	if(!m_dlgTiles.IsWindowVisible())
		pCmdUI->SetCheck(FALSE);
	else
		pCmdUI->SetCheck(TRUE);
}

void CFF4Doc::OnViewTileWindow() 
{
	if(!m_wndTiles.IsWindowVisible())
		m_wndTiles.ShowWindow(SW_SHOW);
	else
		m_wndTiles.ShowWindow(SW_HIDE);
}

void CFF4Doc::OnUpdateViewTileWindow(CCmdUI* pCmdUI) 
{
	if(!m_wndTiles.IsWindowVisible())
		pCmdUI->SetCheck(FALSE);
	else
		pCmdUI->SetCheck(TRUE);
}

void CFF4Doc::OnViewTriggerWindow() 
{
	if(!m_wndTriggers.IsWindowVisible())
		m_wndTriggers.ShowWindow(SW_SHOW);
	else
		m_wndTriggers.ShowWindow(SW_HIDE);
}

void CFF4Doc::OnUpdateViewTriggerWindow(CCmdUI* pCmdUI) 
{
	if(!m_wndTriggers.IsWindowVisible())
		pCmdUI->SetCheck(FALSE);
	else
		pCmdUI->SetCheck(TRUE);
}

void CFF4Doc::OnViewGenericPropWindow() 
{
	if(!m_wndGenericProp.IsWindowVisible())
		m_wndGenericProp.ShowWindow(SW_SHOW);
	else
		m_wndGenericProp.ShowWindow(SW_HIDE);
}

void CFF4Doc::OnUpdateViewGenericPropWindow(CCmdUI* pCmdUI) 
{
	if(!m_wndGenericProp.IsWindowVisible())
		pCmdUI->SetCheck(FALSE);
	else
		pCmdUI->SetCheck(TRUE);
}
