// FF4Edit.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "FF4Edit.h"

#include "MainFrame.h"
#include "FF4Doc.h"
#include "FF4TreeView.h"
#include "OptionsSheet.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// AppSettings

AppSettings::AppSettings()
{
	// Read all the settings at construction so they'll be ready
	// throughout the run of the program.

	bAdvanced = AfxGetApp()->GetProfileInt("Settings", "AdvancedOptions", FALSE);
	bMapGridlines = AfxGetApp()->GetProfileInt("Settings", "MapGridlines", FALSE);
	
	if(!ExtractWindowSpecs(AfxGetApp()->GetProfileString("Settings", "MainWindowSpecs"), &wsMainWindow))
	{
		wsMainWindow.rect = CRect(10, 10, 500, 400);
		wsMainWindow.bVisible = TRUE;
		wsMainWindow.bMaximized = FALSE;
		wsMainWindow.bMinimized = FALSE;
	}

	if(!ExtractWindowSpecs(AfxGetApp()->GetProfileString("Settings", "TileDialogSpecs"), &wsTileDlg))
	{
		wsTileDlg.rect = CRect(CW_USEDEFAULT, CW_USEDEFAULT, 0, 0);
		wsTileDlg.bVisible = FALSE;
		wsTileDlg.bMaximized = FALSE;
		wsTileDlg.bMinimized = FALSE;
	}

	if(!ExtractWindowSpecs(AfxGetApp()->GetProfileString("Settings", "TileWindowSpecs"), &wsTileWnd))
	{
		wsTileWnd.rect = CRect(CW_USEDEFAULT, CW_USEDEFAULT, 0, 0);
		wsTileWnd.bVisible = TRUE;
		wsTileWnd.bMaximized = FALSE;
		wsTileWnd.bMinimized = FALSE;
	}

	if(!ExtractWindowSpecs(AfxGetApp()->GetProfileString("Settings", "TriggerWindowSpecs"), &wsTriggerWnd))
	{
		wsTriggerWnd.rect = CRect(CW_USEDEFAULT, CW_USEDEFAULT, 0, 0);
		wsTriggerWnd.bVisible = FALSE;
		wsTriggerWnd.bMaximized = FALSE;
		wsTriggerWnd.bMinimized = FALSE;
	}

	if(!ExtractWindowSpecs(AfxGetApp()->GetProfileString("Settings", "GenPropWindowSpecs"), &wsGenericPropWnd))
	{
		wsGenericPropWnd.rect = CRect(CW_USEDEFAULT, CW_USEDEFAULT, 0, 0);
		wsGenericPropWnd.bVisible = TRUE;
		wsGenericPropWnd.bMaximized = FALSE;
		wsGenericPropWnd.bMinimized = FALSE;
	}
}

AppSettings::~AppSettings()
{
	// Likewise, store everything at destruction, when the app
	// is closing.

	AfxGetApp()->WriteProfileInt("Settings", "AdvancedOptions", bAdvanced);
	AfxGetApp()->WriteProfileInt("Settings", "MapGridlines", bMapGridlines);

	AfxGetApp()->WriteProfileString("Settings", "MainWindowSpecs", FormatWindowSpecs(&wsMainWindow));
	AfxGetApp()->WriteProfileString("Settings", "TileDialogSpecs", FormatWindowSpecs(&wsTileDlg));
	AfxGetApp()->WriteProfileString("Settings", "TileWindowSpecs", FormatWindowSpecs(&wsTileWnd));
	AfxGetApp()->WriteProfileString("Settings", "TriggerWindowSpecs", FormatWindowSpecs(&wsTriggerWnd));
	AfxGetApp()->WriteProfileString("Settings", "GenPropWindowSpecs", FormatWindowSpecs(&wsGenericPropWnd));
}

CString AppSettings::FormatWindowSpecs(WINDOWSPECS* ws)
{
	CString str;
	str.Format("%04d %04d %04d %04d %1d %1d %1d",
		ws->rect.left, ws->rect.top, ws->rect.right, ws->rect.bottom, ws->bVisible,
		ws->bMaximized, ws->bMinimized);

	return str;
}

BOOL AppSettings::ExtractWindowSpecs(CString& str, WINDOWSPECS* ws)
{
	if(str.IsEmpty())
	{
		return FALSE;
	}
	else
	{
		//******************************************************************
		// ADD UNICODE SUPPORT HERE LATER USING STRTOUL()/WCSTOUL() INSTEAD

		ws->rect.left = atoi((const char*)str);
		ws->rect.top = atoi((const char*)str + 5);
		ws->rect.right = atoi((const char*)str + 10);
		ws->rect.bottom = atoi((const char*)str + 15);
		ws->bVisible = atoi((const char*)str + 20);
		ws->bMaximized = atoi((const char*)str + 22);
		ws->bMinimized = atoi((const char*)str + 24);

		return TRUE;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CFF4EditApp

BEGIN_MESSAGE_MAP(CFF4EditApp, CWinApp)
	//{{AFX_MSG_MAP(CFF4EditApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFF4EditApp construction

CFF4EditApp::CFF4EditApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CFF4EditApp object

CFF4EditApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CFF4EditApp initialization

BOOL CFF4EditApp::InitInstance()
{
	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	SetRegistryKey("YouseiApps");
	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	// Create our settings object here.  It will be destroyed in ExitInstance().
	_Settings = new AppSettings;

	AfxInitRichEdit();
	AfxOleInit();

	// Register clipboard formats.
	clipboardWorldMap = ::RegisterClipboardFormat(_T("FF2ED_WORLDMAP"));

	INITCOMMONCONTROLSEX iccex;
	iccex.dwSize = sizeof(INITCOMMONCONTROLSEX);
	iccex.dwICC = ICC_PAGESCROLLER_CLASS;
	InitCommonControlsEx(&iccex);

	// Initialize DirectX.
	HRESULT rval;
	LPDIRECTDRAW lpDD;

	rval = DirectDrawCreate(NULL, &lpDD, NULL);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	rval = lpDD->QueryInterface(IID_IDirectDraw4, (LPVOID*)&DDraw.lpDD);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	RELEASE(lpDD);

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.
	CSingleDocTemplate* pDocTemplate;
	pDocTemplate = new CSingleDocTemplate(
		IDR_MAINFRAME,
		RUNTIME_CLASS(CFF4Doc),
		RUNTIME_CLASS(CMainFrame),       // main SDI frame window
		RUNTIME_CLASS(CFF4TreeView));
	AddDocTemplate(pDocTemplate);

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CFF4EditApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CFF4EditApp commands

int CFF4EditApp::ExitInstance() 
{
	RELEASE(DDraw.lpDD);
	
	if(_Settings) delete _Settings;

	return CWinApp::ExitInstance();
}

void CFF4EditApp::OnProperties()
{
	// TODO: The property sheet attached to your project
	// via this function is not hooked up to any message
	// handler.  In order to actually use the property sheet,
	// you will need to associate this function with a control
	// in your project such as a menu item or tool bar button.

	COptionsSheet propSheet;

	propSheet.DoModal();

	// This is where you would retrieve information from the property
	// sheet if propSheet.DoModal() returned IDOK.  We aren't doing
	// anything for simplicity.
}
