// FF4TreeView.cpp : implementation of the CFF4TreeView class
//

#include "stdafx.h"
#include "FF4Edit.h"

#include "FF4Doc.h"
#include "FF4TreeView.h"

#include "MainFrame.h"
#include "MapView.h"
#include "LocationView.h"
#include "EnemyView.h"
#include "EventsFormView.h"
#include "EditorInfoView.h"
#include "WorldEnemyView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFF4TreeView

IMPLEMENT_DYNCREATE(CFF4TreeView, CTreeView)

BEGIN_MESSAGE_MAP(CFF4TreeView, CTreeView)
	//{{AFX_MSG_MAP(CFF4TreeView)
	ON_NOTIFY_REFLECT(TVN_SELCHANGED, OnSelChanged)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFF4TreeView construction/destruction

CFF4TreeView::CFF4TreeView()
{
	m_hWorldMaps = NULL;
	m_hLocationMaps1 = NULL;
	m_hLocationMaps2 = NULL;
	m_hEnemies = NULL;
	m_hEvents = NULL;
}

CFF4TreeView::~CFF4TreeView()
{
}

BOOL CFF4TreeView::PreCreateWindow(CREATESTRUCT& cs)
{
	cs.style |= TVS_HASLINES | TVS_HASBUTTONS | TVS_LINESATROOT | TVS_SHOWSELALWAYS;

	return CTreeView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CFF4TreeView drawing

void CFF4TreeView::OnDraw(CDC* pDC)
{
	CFF4Doc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
}

void CFF4TreeView::OnInitialUpdate()
{
	CTreeView::OnInitialUpdate();

	// TODO: You may populate your TreeView with items by directly accessing
	//  its tree control through a call to GetTreeCtrl().
}

/////////////////////////////////////////////////////////////////////////////
// CFF4TreeView diagnostics

#ifdef _DEBUG
void CFF4TreeView::AssertValid() const
{
	CTreeView::AssertValid();
}

void CFF4TreeView::Dump(CDumpContext& dc) const
{
	CTreeView::Dump(dc);
}

CFF4Doc* CFF4TreeView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CFF4Doc)));
	return (CFF4Doc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CFF4TreeView message handlers

HTREEITEM CFF4TreeView::GetNextItem(HTREEITEM hItem)
{
	HTREEITEM hti;

	if(GetTreeCtrl().ItemHasChildren(hItem))
		return GetTreeCtrl().GetChildItem(hItem);		// return first child
	else
	{
		// return next sibling item
		// Go up the tree to find a parent's sibling if needed.
		while((hti = GetTreeCtrl().GetNextSiblingItem(hItem)) == NULL)
		{
			if((hItem = GetTreeCtrl().GetParentItem(hItem)) == NULL)
				return NULL;
		}
	}

	return hti;
}

void CFF4TreeView::ExpandBranch(HTREEITEM hti)
{
	if(GetTreeCtrl().ItemHasChildren(hti))
	{
		GetTreeCtrl().Expand(hti, TVE_EXPAND);
		hti = GetTreeCtrl().GetChildItem(hti);
		
		do ExpandBranch(hti); while((hti = GetTreeCtrl().GetNextSiblingItem(hti)) != NULL);
	}
    
	GetTreeCtrl().EnsureVisible(GetTreeCtrl().GetSelectedItem());
}

void CFF4TreeView::ExpandAll()
{
	HTREEITEM htiCur = GetTreeCtrl().GetRootItem();

	while(htiCur)
	{
		ExpandBranch(htiCur);
		htiCur = GetNextItem(htiCur);
	}
}

void CFF4TreeView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CFF4Doc* pDoc = GetDocument();

	if(!pDoc->m_bValid)
	{
		GetTreeCtrl().SelectItem(NULL);

		m_hWorldMaps = NULL;
		m_hLocationMaps1 = NULL;
		m_hLocationMaps2 = NULL;
		m_hEnemies = NULL;
		m_hEvents = NULL;

		GetTreeCtrl().DeleteAllItems();
		GetTreeCtrl().InsertItem("No file loaded");

		return;
	}

	HTREEITEM sel = GetTreeCtrl().GetSelectedItem();
	GetTreeCtrl().DeleteAllItems();

	m_hWorldMaps = GetTreeCtrl().InsertItem("World Maps");
	for(int i = 0; i < 3; i++)
	{
		HTREEITEM hItem = GetTreeCtrl().InsertItem(pDoc->worldData[i].name, m_hWorldMaps);
		GetTreeCtrl().SetItemData(hItem, (DWORD)&pDoc->worldData[i]);

		HTREEITEM hSubItem = GetTreeCtrl().InsertItem(_T("Enemy Layout"), hItem);
		GetTreeCtrl().SetItemData(hSubItem, (DWORD)&pDoc->worldData[i]);
	}

/*	m_hLocationMaps1 = GetTreeCtrl().InsertItem("Location Maps, Bank 1");
	for(i = 0; i < pDoc->locations.GetSize(); i++)
	{
		HTREEITEM hItem = GetTreeCtrl().InsertItem(pDoc->locations[i]->name, m_hLocationMaps1);
		GetTreeCtrl().SetItemData(hItem, (DWORD)pDoc->locations[i]);
	}
*/
/*	m_hEnemies = GetTreeCtrl().InsertItem("Enemies");
	for(i = 0; i < 256; i++)
	{
		CString s;
		for(int j = 0; j < 8; j++)
		{
			if(pDoc->enemies[i].name[j] >= 0x42 && pDoc->enemies[i].name[j] <= 0x5B)
				s += pDoc->enemies[i].name[j] - 0x01;
			else if(pDoc->enemies[i].name[j] >= 0x5C && pDoc->enemies[i].name[j] <= 0x75)
				s += pDoc->enemies[i].name[j] + 0x05;
			else if(pDoc->enemies[i].name[j] == 0xFF)
				s += " ";
			else
				s += ".";
		}

		HTREEITEM hItem = GetTreeCtrl().InsertItem(s, m_hEnemies);
		GetTreeCtrl().SetItemData(hItem, (DWORD)&pDoc->enemies[i]);
	}
*/
	m_hEvents = GetTreeCtrl().InsertItem("Events");
	GetTreeCtrl().SetItemData(m_hEvents, (DWORD)pDoc->events);

	ExpandAll();

	GetTreeCtrl().SelectItem(sel);
}

void CFF4TreeView::OnSelChanged(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

	HTREEITEM hSel = GetTreeCtrl().GetSelectedItem();

	CFF4Doc* pDoc = GetDocument();

	if(!hSel) return;
	if(!pDoc->m_bValid) return;

	DWORD dwData = GetTreeCtrl().GetItemData(hSel);
	HTREEITEM hParent = GetTreeCtrl().GetParentItem(hSel);
	HTREEITEM hDblParent = GetTreeCtrl().GetParentItem(hParent);

	FLEXPANE fp;
	ZeroMemory(&fp, sizeof(FLEXPANE));

	if(hSel == m_hWorldMaps)
	{
		fp.nID = 0;
		fp.pClass = RUNTIME_CLASS(CEditorInfoView);
		fp.dwData = 0;
		AfxGetMainWnd()->SendMessage(WM_UPDATEFLEXPANE, (UINT)&fp);
	}
	else if(hSel == m_hLocationMaps1)
	{
		fp.nID = 0;
		fp.pClass = RUNTIME_CLASS(CEditorInfoView);
		fp.dwData = 0;
		AfxGetMainWnd()->SendMessage(WM_UPDATEFLEXPANE, (UINT)&fp);
	}
	else if(hParent == m_hWorldMaps)
	{
		fp.nID = IDR_MAPVIEW;
		fp.pClass = RUNTIME_CLASS(CMapView);
		fp.dwData = dwData;
		AfxGetMainWnd()->SendMessage(WM_UPDATEFLEXPANE, (UINT)&fp);
	}
	else if(hDblParent == m_hWorldMaps)
	{
		fp.nID = IDR_MAINFRAME;
		fp.pClass = RUNTIME_CLASS(CWorldEnemyView);
		fp.dwData = dwData;
		AfxGetMainWnd()->SendMessage(WM_UPDATEFLEXPANE, (UINT)&fp);
	}
/*	else if(hParent == m_hLocationMaps1)
	{
		fp.nID = IDR_LOCATIONVIEW;
		fp.pClass = RUNTIME_CLASS(CLocationView);
		fp.dwData = dwData;
		AfxGetMainWnd()->SendMessage(WM_UPDATEFLEXPANE, (UINT)&fp);
	}
/*	else if(hParent == m_hEnemies)
	{
		fp.nID = 0;
		fp.pClass = RUNTIME_CLASS(CEnemyView);
		fp.dwData = dwData;
		AfxGetMainWnd()->SendMessage(WM_UPDATEFLEXPANE, (UINT)&fp);
	}*/
	else if(hSel == m_hEvents)
	{
		fp.nID = 0;
		fp.pClass = RUNTIME_CLASS(CEventsFormView);
		fp.dwData = dwData;
		AfxGetMainWnd()->SendMessage(WM_UPDATEFLEXPANE, (UINT)&fp);
	}

	*pResult = 0;
}

BOOL CFF4TreeView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) 
{
	CWnd* pPane = ((CMainFrame*)AfxGetMainWnd())->m_wndSplitter.GetPane(0, 1);

	if((pPane != NULL) && pPane->OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
      return TRUE;

	return CTreeView::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}
