// LocationView.cpp : implementation of the CLocationView class
//

#include "stdafx.h"
#include "FF4Edit.h"

#include "FF4Doc.h"
#include "LocationView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLocationView

IMPLEMENT_DYNCREATE(CLocationView, CDirectDrawView)

BEGIN_MESSAGE_MAP(CLocationView, CDirectDrawView)
	//{{AFX_MSG_MAP(CLocationView)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_TIMER()
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
	ON_COMMAND(ID_VIEW_GRIDLINES, OnViewGridlines)
	ON_UPDATE_COMMAND_UI(ID_VIEW_GRIDLINES, OnUpdateViewGridlines)
	ON_WM_CONTEXTMENU()
	ON_WM_RBUTTONDOWN()
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_COMMAND(ID_EDIT_CUT, OnEditCut)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_WM_SETCURSOR()
	//}}AFX_MSG_MAP
	ON_COMMAND_RANGE(ID_TOOLS_POINTER, ID_TOOLS_FILL, OnChangeTool)
	ON_UPDATE_COMMAND_UI_RANGE(ID_TOOLS_POINTER, ID_TOOLS_FILL, OnUpdateTool)
	ON_MESSAGE(WM_MOUSELEAVE, OnMouseLeave)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCutOrCopy)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCutOrCopy)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLocationView construction/destruction

CLocationView::CLocationView()
{
	m_locationData = NULL;

	m_ptMouseTile = CPoint(0, 0);
	m_ptCurrentTrigger = CPoint(0, 0);

	m_nRoundDown = 16;

	m_nCurrentTool = ID_TOOLS_POINTER;

	m_rcTool.SetRectEmpty();
	m_rcMarquee.SetRectEmpty();

	m_bTimerSet = FALSE;

	m_rcTracker.m_nStyle = CRectTracker::solidLine | CRectTracker::hatchedBorder;
	m_rcTracker.m_rect.SetRectEmpty();

	m_bClipWidth = m_bClipHeight = 0;
	m_pClipboard = NULL;

	m_ptMarqueeDrag = CPoint(-1, -1);
	m_bPasting = FALSE;
}

CLocationView::~CLocationView()
{
}

BOOL CLocationView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CDirectDrawView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CLocationView drawing


void CLocationView::OnDraw(CDC* pDC)
{
	CClientDC dc(this);

	CFF4Doc* pDoc = GetDocument();

	BeginDraw();

	if(m_locationData == NULL) { EndDraw();  return; }

	CRect clientRect;  GetClientRect(&clientRect);
	int width = m_rcUpdate.Width();
	int height = m_rcUpdate.Height();

	int xBegin = 0, xEnd = 0, yBegin = 0, yEnd = 0, xTileBegin = 0, yTileBegin = 0;

	xTileBegin = GetScrollPosition().x;
	yTileBegin = GetScrollPosition().y;
	xBegin = m_rcUpdate.left / 16 + GetScrollPosition().x;
	yBegin = m_rcUpdate.top / 16 + GetScrollPosition().y;
	xEnd = min(xBegin + width / 16 + 1, m_locationData->gLocationData->mapSize.cx);
	yEnd = min(yBegin + height / 16 + 1, m_locationData->gLocationData->mapSize.cy);

	DDSURFACEDESC2 ddsd;
	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	HRESULT hres;

	if(!pDoc->m_bValid)
	{
		EndDraw();
		return;
	}

	while((hres = m_lpSurfaceBack->Lock(NULL, &ddsd, 0, NULL)) == DDERR_WASSTILLDRAWING);
	if(hres == DD_OK)
	{
		for(int y = yBegin; y < yEnd; y++)
		{
			for(int x = xBegin; x < xEnd; x++)
			{
				Put16x16Tile(&ddsd, (x - xBegin) * 16, (y - yBegin) * 16,
					m_locationData->GetMapTile(x, y));

				if(m_nCurrentTool == ID_TOOLS_POINTER)
				{
					Entrance* pEntr = m_locationData->FindEntranceFromSource(x, y);

					if(pEntr)
					{
						ShadeRect(&ddsd, (x - xBegin) * 16, (y - yBegin) * 16,
							16, 16, DDColor(15, 15, 31), 60);
					}
					
					if(!pEntr && pDoc->gLocationData->tileProperties[m_locationData->tileSet]
						[m_locationData->GetMapTile(x, y)] & LDF_ENTRANCE)
					{
						ShadeRect(&ddsd, (x - xBegin) * 16, (y - yBegin) * 16,
							16, 16, DDColor(15, 31, 31), 60);
					}

					if(pDoc->gLocationData->tileProperties[m_locationData->tileSet]
						[m_locationData->GetMapTile(x, y)] & LDF_EXIT)
					{
						ShadeRect(&ddsd, (x - xBegin) * 16, (y - yBegin) * 16,
							16, 16, DDColor(31, 15, 31), 60);
					}
				}

				if(x == m_ptCurrentTrigger.x && y == m_ptCurrentTrigger.y)
				{
					Entrance* pEntr = m_locationData->FindEntranceFromSource(x, y);

					if(pEntr)
					{
						ShadeRect(&ddsd, (x - xBegin) * 16, (y - yBegin) * 16,
							16, 16, DDColor(31, 31, 15), 60);
					}
				}
			}
		}

		m_lpSurfaceBack->Unlock(NULL);
	}

	DoTool(MTA_DRAWNOLOCK, NULL);

	if(Settings.bMapGridlines)
	{
		for(int x = 0; x < min(4096, (m_locationData->gLocationData->mapSize.cx + 1) * 16); x += 16)
			VerticalLine(x, 0, min(clientRect.bottom, 32 * 16),
			DDColor(24, 24, 24));

		for(int y = 0; y < min(4096, (m_locationData->gLocationData->mapSize.cy + 1) * 16); y += 16)
			HorizontalLine(0, y, min(clientRect.right, 32 * 16),
			DDColor(24, 24, 24));
	}

	EndDraw();

	if(!m_rcMarquee.IsRectEmpty())
	{
		m_rcTracker.m_rect.left = (m_rcMarquee.left - GetScrollPosition().x) * 16;
		m_rcTracker.m_rect.top = (m_rcMarquee.top - GetScrollPosition().y) * 16;
		m_rcTracker.m_rect.right = (m_rcMarquee.right - GetScrollPosition().x) * 16;
		m_rcTracker.m_rect.bottom = (m_rcMarquee.bottom - GetScrollPosition().y) * 16;
		m_rcTracker.Draw(&dc);
	}
	else
	{
		m_rcTracker.m_rect.SetRectEmpty();
	}

	TOOLPACKAGE tp;
	tp.pDC = &dc;
	DoTool(MTA_DRAWAFTERFLIP, &tp);
}

void CLocationView::Put8x8x3Tile(DDSURFACEDESC2* ddsd, int dx, int dy, LocationTilePart* pTilePart)
{
	CFF4Doc* pDoc = GetDocument();

	BYTE aTile[64];
	BYTE bPlanes[3];

	BYTE* pTile = pDoc->gLocationData->tiles[m_locationData->tileSet] + pTilePart->tilePart * 24;

	for(int row = 0; row < 8; row++)
	{
		bPlanes[0] = pTile[row * 2];
		bPlanes[1] = pTile[row * 2 + 1];
		bPlanes[2] = pTile[row + 16];

		for(int bit = 7; bit >= 0; bit--)
		{
			BYTE mask = 1 << bit;

			// Get a color index by separating the bits on each plane,
			// shifting them the correct amount depending on the plane,
			// and ORing them together.
			aTile[row * 8 + (7 - bit)] = ((bPlanes[0] & mask) >> bit) |
								   (((bPlanes[1] & mask) >> bit) << 1) |
								   (((bPlanes[2] & mask) >> bit) << 2);
								   
		}
	}

	int xSrc, ySrc;

	if(pTilePart->palette & 0x80) ySrc = 7; else ySrc = 0;

	DWORD c1f;
	BYTE b1, r, g, b;
	WORD c555;

	if(DDraw.ddpfPrimary.dwRGBBitCount == 16)
	{
		WORD* lpDest = (WORD*)ddsd->lpSurface + (ddsd->lPitch / 2) * dy;

		for(int y = dy; y < dy + 8; y++)
		{
			if(pTilePart->palette & 0x40) xSrc = 7; else xSrc = 0;

			for(int x = dx; x < dx + 8; x++)
			{
				b1 = aTile[ySrc * 8 + xSrc];

				c555 = pDoc->gLocationData->palettes[m_locationData->paletteIndex][(pTilePart->palette & 0x1F) * 2 + b1];
				r = R555(c555);
				g = G555(c555);
				b = B555(c555);
				c1f = (r) | (g << 6) | (b << 11);

				lpDest[x] = (WORD)c1f;

				if(pTilePart->palette & 0x40) xSrc--; else xSrc++;
			}

			if(pTilePart->palette & 0x80) ySrc--; else ySrc++;

			lpDest += ddsd->lPitch / 2;
		}
	}
	else if(DDraw.ddpfPrimary.dwRGBBitCount == 24)
	{
		BYTE* lpDest = (BYTE*)ddsd->lpSurface + (ddsd->lPitch) * dy;

		for(int y = dy; y < dy + 8; y++)
		{
			if(pTilePart->palette & 0x40) xSrc = 7; else xSrc = 0;
			int sx = dx * 3;

			for(int x = dx; x < dx + 8; x++)
			{
				b1 = aTile[ySrc * 8 + xSrc];

				c555 = pDoc->gLocationData->palettes[m_locationData->paletteIndex][(pTilePart->palette & 0x1F) * 2 + b1];
				r = R555(c555) * 8;
				g = G555(c555) * 8;
				b = B555(c555) * 8;
				c1f = (r) | (g << 6) | (b << 11);

				lpDest[sx] = r;
				lpDest[sx + 1] = g;
				lpDest[sx + 2] = b;

				sx += 6;

				if(pTilePart->palette & 0x40) xSrc--; else xSrc++;
			}

			if(pTilePart->palette & 0x80) ySrc--; else ySrc++;

			lpDest += ddsd->lPitch;
		}
	}
	else if(DDraw.ddpfPrimary.dwRGBBitCount == 32)
	{
		DWORD* lpDest = (DWORD*)ddsd->lpSurface + (ddsd->lPitch / 4) * dy;

		for(int y = dy; y < dy + 8; y++)
		{
			if(pTilePart->palette & 0x40) xSrc = 7; else xSrc = 0;

			for(int x = dx; x < dx + 8; x++)
			{
				b1 = aTile[ySrc * 8 + xSrc];

				c555 = pDoc->gLocationData->palettes[m_locationData->paletteIndex][(pTilePart->palette & 0x1F) * 2 + b1];
				r = R555(c555) * 8;
				g = G555(c555) * 8;
				b = B555(c555) * 8;
				c1f = (r) | (g << 8) | (b << 16);

				lpDest[x] = c1f;

				if(pTilePart->palette & 0x40) xSrc--; else xSrc++;
			}

			if(pTilePart->palette & 0x80) ySrc--; else ySrc++;

			lpDest += ddsd->lPitch / 4;
		}
	}
}

void CLocationView::Put8x8x4Tile(DDSURFACEDESC2* ddsd, int dx, int dy, LocationTilePart* pTilePart)
{
	CFF4Doc* pDoc = GetDocument();

	BYTE aTile[64];
	BYTE bPlanes[4];

	BYTE* pTile = pDoc->gLocationData->tiles[m_locationData->tileSet] + pTilePart->tilePart * 24;

	for(int row = 0; row < 8; row++)
	{
		bPlanes[0] = pTile[row * 2];
		bPlanes[1] = pTile[row * 2 + 1];
		bPlanes[2] = pTile[row * 2 + 16];
		bPlanes[3] = pTile[row * 2 + 17];

		for(int bit = 7; bit >= 0; bit--)
		{
			BYTE mask = 1 << bit;

			// Get a color index by separating the bits on each plane,
			// shifting them the correct amount depending on the plane,
			// and ORing them together.
//			aTile[row * 8 + (7 - bit)] = (bPlanes[0] & mask) |
//								   ((bPlanes[1] & mask) << 1) |
//								   ((bPlanes[2] & mask) << 2) |
//								   ((bPlanes[3] & mask) << 3);
			aTile[row * 8 + (7 - bit)] = ((bPlanes[0] & mask) >> bit) |
								   (((bPlanes[1] & mask) >> bit) << 1) |
								   (((bPlanes[2] & mask) >> bit) << 2);
								   (((bPlanes[3] & mask) >> bit) << 3);
		}
	}

	int xSrc, ySrc;

	if(pTilePart->palette & 0x80) ySrc = 7; else ySrc = 0;

	DWORD c1f;
	BYTE b1, r, g, b;
	WORD c555;

	if(DDraw.ddpfPrimary.dwRGBBitCount == 16)
	{
		WORD* lpDest = (WORD*)ddsd->lpSurface + (ddsd->lPitch / 2) * dy;

		for(int y = dy; y < dy + 8; y++)
		{
			if(pTilePart->palette & 0x40) xSrc = 7; else xSrc = 0;

			for(int x = dx; x < dx + 8; x++)
			{
				b1 = aTile[ySrc * 8 + xSrc];

				c555 = pDoc->gLocationData->palettes[m_locationData->paletteIndex][(pTilePart->palette & 0x1F) * 2 + b1];
				r = R555(c555);
				g = G555(c555);
				b = B555(c555);
				c1f = (r) | (g << 6) | (b << 11);

				lpDest[x] = (WORD)c1f;

				if(pTilePart->palette & 0x40) xSrc--; else xSrc++;
			}

			if(pTilePart->palette & 0x80) ySrc--; else ySrc++;

			lpDest += ddsd->lPitch / 2;
		}
	}
	else if(DDraw.ddpfPrimary.dwRGBBitCount == 24)
	{
		BYTE* lpDest = (BYTE*)ddsd->lpSurface + (ddsd->lPitch) * dy;

		for(int y = dy; y < dy + 8; y++)
		{
			if(pTilePart->palette & 0x40) xSrc = 7; else xSrc = 0;
			int sx = dx * 3;

			for(int x = dx; x < dx + 8; x++)
			{
				b1 = aTile[ySrc * 8 + xSrc];

				c555 = pDoc->gLocationData->palettes[m_locationData->paletteIndex][(pTilePart->palette & 0x1F) * 2 + b1];
				r = R555(c555) * 8;
				g = G555(c555) * 8;
				b = B555(c555) * 8;
				c1f = (r) | (g << 6) | (b << 11);

				lpDest[sx] = r;
				lpDest[sx + 1] = g;
				lpDest[sx + 2] = b;

				sx += 6;

				if(pTilePart->palette & 0x40) xSrc--; else xSrc++;
			}

			if(pTilePart->palette & 0x80) ySrc--; else ySrc++;

			lpDest += ddsd->lPitch;
		}
	}
	else if(DDraw.ddpfPrimary.dwRGBBitCount == 32)
	{
		DWORD* lpDest = (DWORD*)ddsd->lpSurface + (ddsd->lPitch / 4) * dy;

		for(int y = dy; y < dy + 8; y++)
		{
			if(pTilePart->palette & 0x40) xSrc = 7; else xSrc = 0;

			for(int x = dx; x < dx + 8; x++)
			{
				b1 = aTile[ySrc * 8 + xSrc];

				c555 = pDoc->gLocationData->palettes[m_locationData->paletteIndex][(pTilePart->palette & 0x1F) * 2 + b1];
				r = R555(c555) * 8;
				g = G555(c555) * 8;
				b = B555(c555) * 8;
				c1f = (r) | (g << 8) | (b << 16);

				lpDest[x] = c1f;

				if(pTilePart->palette & 0x40) xSrc--; else xSrc++;
			}

			if(pTilePart->palette & 0x80) ySrc--; else ySrc++;

			lpDest += ddsd->lPitch / 4;
		}
	}
}

void CLocationView::Put16x16Tile(DDSURFACEDESC2* ddsd, int dx, int dy, BYTE bTile)
{
	if(bTile > 0x7F) return;

	CFF4Doc* pDoc = GetDocument();

	if(m_locationData->tileSet == 0x00 || m_locationData->tileSet == 0x0F)
	{
		Put8x8x4Tile(ddsd, dx, dy, &pDoc->gLocationData->tileFormations[m_locationData->tileSet][bTile + 0]);
		Put8x8x4Tile(ddsd, dx + 8, dy,  &pDoc->gLocationData->tileFormations[m_locationData->tileSet][bTile + 128]);
		Put8x8x4Tile(ddsd, dx, dy + 8,  &pDoc->gLocationData->tileFormations[m_locationData->tileSet][bTile + 256]);
		Put8x8x4Tile(ddsd, dx + 8, dy + 8,  &pDoc->gLocationData->tileFormations[m_locationData->tileSet][bTile + 384]);
	}
	else
	{
		Put8x8x3Tile(ddsd, dx, dy, &pDoc->gLocationData->tileFormations[m_locationData->tileSet][bTile + 0]);
		Put8x8x3Tile(ddsd, dx + 8, dy,  &pDoc->gLocationData->tileFormations[m_locationData->tileSet][bTile + 128]);
		Put8x8x3Tile(ddsd, dx, dy + 8,  &pDoc->gLocationData->tileFormations[m_locationData->tileSet][bTile + 256]);
		Put8x8x3Tile(ddsd, dx + 8, dy + 8,  &pDoc->gLocationData->tileFormations[m_locationData->tileSet][bTile + 384]);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CLocationView diagnostics

#ifdef _DEBUG
void CLocationView::AssertValid() const
{
	CDirectDrawView::AssertValid();
}

void CLocationView::Dump(CDumpContext& dc) const
{
	CDirectDrawView::Dump(dc);
}

CFF4Doc* CLocationView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CFF4Doc)));
	return (CFF4Doc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CLocationView message handlers

void CLocationView::OnInitialUpdate() 
{
	CDirectDrawView::OnInitialUpdate();
	
	CSize sizeTotal, sizePage;
	sizeTotal.cx = sizeTotal.cy = 0;
	sizePage.cx = sizePage.cy = 16;
	SetScrollSizes(sizeTotal, sizePage);
}

void CLocationView::OnViewGridlines() 
{
	if(Settings.bMapGridlines == TRUE) Settings.bMapGridlines = FALSE;
	else Settings.bMapGridlines = TRUE;

	Invalidate();
}

void CLocationView::OnUpdateViewGridlines(CCmdUI* pCmdUI) 
{
	if(Settings.bMapGridlines == TRUE) pCmdUI->SetCheck(TRUE);
	else pCmdUI->SetCheck(FALSE);	
}

PROPERTYITEM g_strDirections[] =
{
	{ DIRECTION_UP, "Up", NULL },
	{ DIRECTION_RIGHT, "Right", NULL },
	{ DIRECTION_DOWN, "Down", NULL },
	{ DIRECTION_LEFT, "Left", NULL },
	{ -1, NULL, NULL },
};

PROPERTYITEM g_strLocBattleBg[] =
{
	{ 0, "Grass", NULL },
	{ 1, "Forest", NULL },
	{ 2, "Mountain", NULL },
	{ 3, "Castle or Building", NULL },
	{ 4, "Brown Cave", NULL },
	{ 5, "Moon Surface", NULL },
	{ 6, "Ship Deck", NULL },
	{ 7, "Watery Cave", NULL },
	{ 8, "Grey Cave", NULL },
	{ 9, "Desert", NULL },
	{ 10, "Shore", NULL },
	{ 11, "Underground Cave", NULL },
	{ 12, "Mechanical Tower", NULL },
	{ 13, "Lunar Cave", NULL },
	{ 14, "Crystal Room", NULL },
	{ 15, "Underground", NULL },
	{ -1, NULL, NULL },
};

void CLocationView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CFF4Doc* pDoc = GetDocument();

	if(lHint == NULL || (LocationData*)lHint == m_locationData) return;

	m_locationData = (LocationData*)lHint;

	// Update the tile list.
	pDoc->m_wndTiles.m_wndTileList.ResetContent();
	for(int i = 0; i < 128; i++)
	{
		int nIndex = pDoc->m_wndTiles.m_wndTileList.AddString(NULL);
		pDoc->m_wndTiles.m_wndTileList.SetItemData(nIndex, (DWORD)m_locationData);
	}

	// Update the property tree.
/*	pDoc->m_wndGenericProp.m_locationData = m_locationData;

	pDoc->m_wndGenericProp.m_wndPropTree.DeleteAllItems();
	pDoc->m_wndGenericProp.m_wndPropTree.SetTrueFalseText(_T("Yes"), _T("No"));
	pDoc->m_wndGenericProp.m_wndPropTree.AddGroup(1000, "Appearance");
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(1000, 10, "Map Index", PROPT_NUMEDIT, 0, m_locationData->mapIndex);
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(1000, 20, "Tile Set", PROPT_NUMEDIT, 0, m_locationData->tileSet);
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(1000, 40, "Surrounding Tile", PROPT_NUMEDIT, 0, m_locationData->borderTile);
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(1000, 50, "Palette Index", PROPT_NUMEDIT, 0, m_locationData->paletteIndex);
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(1000, 80, "Background Tile", PROPT_NUMEDIT, 0, m_locationData->backgroundTile);
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(1000, 90, "Background Scroll Direction", PROPT_COMBO, 0, (LPARAM)&g_strDirections, (m_locationData->scrollSpecs & SSF_DIRECTION) >> 4);
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(1000, 91, "Background Scroll Speed", PROPT_NUMEDIT, 0, (m_locationData->scrollSpecs & SSF_SPEED) >> 6);
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(1000, 92, "Sticky Background, Vertical", PROPT_BOOLEAN, 0, (m_locationData->scrollSpecs & SSF_VERTSTICKY) ? TRUE : FALSE);
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(1000, 93, "Sticky Background, Horizontal", PROPT_BOOLEAN, 0, (m_locationData->scrollSpecs & SSF_HORZSTICKY) ? TRUE : FALSE);
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(1000, 94, "Transparent Background", PROPT_BOOLEAN, 0, (m_locationData->scrollSpecs & SSF_TRANSPARENT) ? TRUE : FALSE);
	pDoc->m_wndGenericProp.m_wndPropTree.AddGroup(2000, "Events");
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(2000, 30, "NPC Index", PROPT_NUMEDIT, 0, m_locationData->NPCIndex);
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(2000, 60, "NPC Palette Index", PROPT_NUMEDIT, 0, m_locationData->NPCPaletteIndex);
	pDoc->m_wndGenericProp.m_wndPropTree.AddGroup(3000, "Miscellaneous");
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(3000, 70, "Song", PROPT_NUMEDIT, 0, m_locationData->musicIndex);
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(3000, 110, "Title", PROPT_NUMEDIT, 0, m_locationData->titleIndex);
	pDoc->m_wndGenericProp.m_wndPropTree.AddGroup(4000, "Battles");
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(4000, 0, "Battle Background", PROPT_COMBO, 0, (LPARAM)&g_strLocBattleBg, m_locationData->battleBackground & LD_BATTLEBACKGROUND);
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(4000, 1, "Use Alternate Palette", PROPT_BOOLEAN, 0, (m_locationData->battleBackground & LD_BATTLEALTPALETTE) ? TRUE : FALSE);
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(4000, 2, "Battle is Magnetic", PROPT_BOOLEAN, 0, (m_locationData->battleBackground & LD_BATTLEMAGNETIC) ? TRUE : FALSE);
	pDoc->m_wndGenericProp.m_wndPropTree.AddGroup(5000, "Unknowns");
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(5000, 100, "Unknown 1", PROPT_NUMEDIT, 0, m_locationData->UK2);
		pDoc->m_wndGenericProp.m_wndPropTree.AddItem(5000, 120, "Unknown 2", PROPT_NUMEDIT, 0, m_locationData->UK3);

	pDoc->m_wndGenericProp.m_wndPropTree.ExpandAll();
*/
	// Update the trigger list.
/*	CString s, t;
	pDoc->m_wndTriggers.m_locationData = m_worldData;
	pDoc->m_wndTriggers.m_wndTriggerList.ResetContent();
	pDoc->m_wndTriggers.m_wndTriggerList.SetItemHeight(0, 16);

	for(i = 0; i < m_locationData->entrances.GetSize(); i++)
	{
		int nIndex = pDoc->m_wndTriggers.m_wndTriggerList.AddString(NULL);
		pDoc->m_wndTriggers.m_wndTriggerList.SetItemData(nIndex, (DWORD)m_locationData->entrances[i]);
	}
*/
	// Update the tile editing dialog.
/*	pDoc->m_dlgTiles.m_worldData = m_worldData;
	pDoc->m_dlgTiles.m_wndCurrentTile.m_worldData = m_worldData;
	pDoc->m_dlgTiles.m_wndTileList.ResetContent();
	pDoc->m_dlgTiles.m_wndTileList.SetColumnWidth(22);
	pDoc->m_dlgTiles.m_wndTileList.SetItemHeight(0, 22);
	for(i = 0; i < 128; i++)
	{
		int nIndex = pDoc->m_dlgTiles.m_wndTileList.AddString(NULL);
		pDoc->m_dlgTiles.m_wndTileList.SetItemData(nIndex, (DWORD)m_worldData);
	}

	pDoc->m_dlgTiles.m_wndSmallTileList.ResetContent();
	pDoc->m_dlgTiles.m_wndSmallTileList.SetColumnWidth(22);
	pDoc->m_dlgTiles.m_wndSmallTileList.SetItemHeight(0, 22);
	for(i = 0; i < 256; i++)
	{
		int nIndex = pDoc->m_dlgTiles.m_wndSmallTileList.AddString(NULL);
		pDoc->m_dlgTiles.m_wndSmallTileList.SetItemData(nIndex, (DWORD)m_worldData);
	}

	// Update the palette combo.
	pDoc->m_dlgTiles.m_wndPalettes.ResetContent();
	for(i = 0; i < 4; i++)
	{
		CString s;  s.Format(_T("Palette %d"), i);
		pDoc->m_dlgTiles.m_wndPalettes.AddString(s);
	}

	pDoc->m_dlgTiles.m_wndColors.m_worldData = m_worldData;
	pDoc->m_dlgTiles.m_wndTileEdit.m_worldData = m_worldData;

	pDoc->m_dlgTiles.m_wndTileList.SetCurSel(0);
	pDoc->m_dlgTiles.SendCommandMessage(&pDoc->m_dlgTiles.m_wndTileList, LBN_SELCHANGE);
*/
	CSize sizeTotal, sizePage;
	sizeTotal.cx = m_locationData->gLocationData->mapSize.cx;
	sizeTotal.cy = m_locationData->gLocationData->mapSize.cy;
	sizePage.cx = sizePage.cy = 16;
	SetScrollSizes(sizeTotal, sizePage);
	ScrollToPosition(m_locationData->ppdLastScrollPos);

	Invalidate();
}

BOOL CLocationView::OnScroll(UINT nScrollCode, UINT nPos, BOOL bDoScroll) 
{
	BOOL res = CDirectDrawView::OnScroll(nScrollCode, nPos, bDoScroll);

	m_locationData->ppdLastScrollPos = GetScrollPosition();

	return res;
}

void CLocationView::SetCurrentTrigger(CPoint pt)
{
	CPoint oldTrigger = m_ptCurrentTrigger;
	m_ptCurrentTrigger = pt;

	CPoint ptOldScreen((oldTrigger.x - GetScrollPosition().x) * 16,
		(oldTrigger.y - GetScrollPosition().y) * 16);
	CPoint ptScreen((pt.x - GetScrollPosition().x) * 16,
		(pt.y - GetScrollPosition().y) * 16);

	CRect rect;  GetClientRect(rect);

	if(rect.PtInRect(ptOldScreen))
		InvalidateRect(CRect(ptOldScreen.x, ptOldScreen.y, ptOldScreen.x + 16, ptOldScreen.y + 16));
	if(rect.PtInRect(ptScreen))
		InvalidateRect(CRect(ptScreen.x, ptScreen.y, ptScreen.x + 16, ptScreen.y + 16));
}

void CLocationView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if(!m_locationData) return;

	SetCapture();

	if(m_bPasting) return;

	CRect clientRect;  GetClientRect(clientRect);

	TOOLPACKAGE tp;
	tp.xTile = GetScrollPosition().x + point.x / 16;
	if(tp.xTile < 0) tp.xTile = 0;
	else if(tp.xTile > m_locationData->gLocationData->mapSize.cx - 1)
		tp.xTile = m_locationData->gLocationData->mapSize.cx - 1;
	tp.yTile = GetScrollPosition().y + point.y / 16;
	if(tp.yTile < 0) tp.yTile = 0;
	else if(tp.yTile > m_locationData->gLocationData->mapSize.cy - 1)
		tp.yTile = m_locationData->gLocationData->mapSize.cy - 1;
	tp.xScreenTile = point.x / 16;
	if(tp.xScreenTile < 0) tp.xScreenTile = 0;
	else if(tp.xScreenTile > clientRect.Width() / 16) tp.xScreenTile = clientRect.Width() / 16;
	tp.yScreenTile = point.y / 16;
	if(tp.yScreenTile < 0) tp.yScreenTile = 0;
	else if(tp.yScreenTile > clientRect.Height() / 16) tp.yScreenTile = clientRect.Height() / 16;
	DoTool(MTA_ONBUTTONDOWN, &tp);

	CDirectDrawView::OnLButtonDown(nFlags, point);
}

void CLocationView::OnMouseMove(UINT nFlags, CPoint point) 
{
/*	if(!m_locationData) return;

	int xTile = (GetScrollPosition().x + point.x) / 16;
	int yTile = (GetScrollPosition().y + point.y) / 16;

	int xScreenTile = point.x / 16;
	int yScreenTile = point.y / 16;

	CString s;  s.Format("Current position:  X: %d  Y: %d", xTile, yTile);
	
	CPoint ptScreen = CPoint(xScreenTile * 16, yScreenTile * 16);
	ClientToScreen(&ptScreen);

	Entrance* pEntr = m_locationData->FindEntranceFromSource(xTile, yTile);
	if(pEntr)
	{
		CString info;
		info.Format("     Entrance:  Destination: %02X\n(%d, %d)", pEntr->destination, pEntr->destX, pEntr->destY);
		s += info;
	}

	((CFrameWnd*)AfxGetMainWnd())->SetMessageText(s);

	CDirectDrawView::OnMouseMove(nFlags, point);*/
	if(!m_locationData) return;

	CRect clientRect;  GetClientRect(clientRect);

	TOOLPACKAGE tp;
	tp.xTile = GetScrollPosition().x + point.x / 16;
	if(tp.xTile < 0) tp.xTile = 0;
	else if(tp.xTile > m_locationData->gLocationData->mapSize.cx - 1)
		tp.xTile = m_locationData->gLocationData->mapSize.cx - 1;
	tp.yTile = GetScrollPosition().y + point.y / 16;
	if(tp.yTile < 0) tp.yTile = 0;
	else if(tp.yTile > m_locationData->gLocationData->mapSize.cy - 1)
		tp.yTile = m_locationData->gLocationData->mapSize.cy - 1;
	tp.xScreenTile = point.x / 16;
	if(tp.xScreenTile < 0) tp.xScreenTile = 0;
	else if(tp.xScreenTile > clientRect.Width() / 16) tp.xScreenTile = clientRect.Width() / 16;
	tp.yScreenTile = point.y / 16;
	if(tp.yScreenTile < 0) tp.yScreenTile = 0;
	else if(tp.yScreenTile > clientRect.Height() / 16) tp.yScreenTile = clientRect.Height() / 16;

	DoTool(MTA_MOVENOCAPTURE, &tp);

	CString s;  s.Format("Current position:  X: %d  Y: %d", tp.xTile, tp.yTile);
	((CFrameWnd*)AfxGetMainWnd())->SetMessageText(s);

	if(GetCapture())
	{
		DoTool(MTA_ONMOUSEMOVE, &tp);

		CPoint ptLastVisible(clientRect.Width() / 16, clientRect.Height() / 16);

		m_ptScrollOffset = CPoint(0, 0);
		if(point.x < 24) m_ptScrollOffset.x = -1;
		if(point.y < 24) m_ptScrollOffset.y = -1;
		if(point.x > clientRect.right - 24) m_ptScrollOffset.x = 1;
		if(point.y > clientRect.bottom - 24) m_ptScrollOffset.y = 1;

		if(m_ptScrollOffset != CPoint(0, 0))
		{
			if(!m_bTimerSet)
			{
				SetTimer(31337, 10, NULL);
				m_bTimerSet = TRUE;
			}
		}
		else
		{
			KillTimer(31337);
			m_bTimerSet = FALSE;
		}
	}

	m_ptLastMouse = point;

	CDirectDrawView::OnMouseMove(nFlags, point);

	TRACKMOUSEEVENT tme;
	tme.cbSize = sizeof(TRACKMOUSEEVENT);
	tme.dwFlags = TME_LEAVE;
	tme.hwndTrack = GetSafeHwnd();
	_TrackMouseEvent(&tme);
}

void CLocationView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CRect clientRect;  GetClientRect(clientRect);

	TOOLPACKAGE tp;
	tp.xTile = GetScrollPosition().x + point.x / 16;
	if(tp.xTile < 0) tp.xTile = 0;
	else if(tp.xTile > m_locationData->gLocationData->mapSize.cx - 1)
		tp.xTile = m_locationData->gLocationData->mapSize.cx - 1;
	tp.yTile = GetScrollPosition().y + point.y / 16;
	if(tp.yTile < 0) tp.yTile = 0;
	else if(tp.yTile > m_locationData->gLocationData->mapSize.cy - 1)
		tp.yTile = m_locationData->gLocationData->mapSize.cy - 1;
	tp.xScreenTile = point.x / 16;
	if(tp.xScreenTile < 0) tp.xScreenTile = 0;
	else if(tp.xScreenTile > clientRect.Width() / 16) tp.xScreenTile = clientRect.Width() / 16;
	tp.yScreenTile = point.y / 16;
	if(tp.yScreenTile < 0) tp.yScreenTile = 0;
	else if(tp.yScreenTile > clientRect.Height() / 16) tp.yScreenTile = clientRect.Height() / 16;

	if(m_bPasting)
	{
		COleDataObject obj;

		if(obj.AttachClipboard())
		{
			if(obj.IsDataAvailable(App.clipboardWorldMap))
			{
				HGLOBAL hmem = obj.GetGlobalData(App.clipboardWorldMap);
				CMemFile sf((BYTE*)::GlobalLock(hmem), ::GlobalSize(hmem));

				sf.Read(&m_bClipWidth, 1);
				sf.Read(&m_bClipHeight, 1);

				if(m_pClipboard) delete m_pClipboard;
				m_pClipboard = new BYTE[m_bClipWidth * m_bClipHeight];
				sf.Read(m_pClipboard, m_bClipWidth * m_bClipHeight);

				::GlobalUnlock(hmem);

				BOOL bFirst = TRUE;
				for(int y = tp.yTile; y < min(tp.yTile + m_bClipHeight, m_locationData->gLocationData->mapSize.cy); y++)
				{
					for(int x = tp.xTile; x < min(tp.xTile + m_bClipWidth, m_locationData->gLocationData->mapSize.cx); x++)
					{
						AddUndoAction(bFirst, x, y, m_locationData->GetMapTile(x, y));
						bFirst = FALSE;
						m_locationData->SetMapTile(x, y,
							m_pClipboard[(y - tp.yTile) * m_bClipWidth + (x - tp.xTile)]);
					}
				}

				m_ptMarqueeDrag = CPoint(-1, -1);
				m_rcTool.SetRectEmpty();
				m_rcMarquee.SetRectEmpty();
				Invalidate();
				m_bPasting = FALSE;

				ReleaseCapture();
				KillTimer(31337);
				m_bTimerSet = FALSE;
				m_ptScrollOffset = CPoint(0, 0);

				CDirectDrawView::OnLButtonUp(nFlags, point);
				return;
			}
		}
	}

	DoTool(MTA_ONBUTTONUP, &tp);

	ReleaseCapture();
	KillTimer(31337);
	m_bTimerSet = FALSE;
	m_ptScrollOffset = CPoint(0, 0);

	CDirectDrawView::OnLButtonUp(nFlags, point);
}

void CLocationView::DoTool(int nContext, TOOLPACKAGE* tp)
{
	if(m_nCurrentTool == ID_TOOLS_POINTER) DoToolPointer(nContext, tp);
	else if(m_nCurrentTool == ID_TOOLS_PICKUP) DoToolPickup(nContext, tp);
	else if(m_nCurrentTool == ID_TOOLS_MARQUEE) DoToolMarquee(nContext, tp);
	else if(m_nCurrentTool == ID_TOOLS_PENCIL) DoToolPencil(nContext, tp);
	else if(m_nCurrentTool == ID_TOOLS_RECTFILL) DoToolRectFill(nContext, tp);
//	else if(m_nCurrentTool == ID_TOOLS_FILL) DoToolFill(nContext, tp);
}

void CLocationView::DoToolPencil(int nContext, TOOLPACKAGE* tp)
{
	CFF4Doc* pDoc = GetDocument();
	int bTile = pDoc->m_wndTiles.m_wndTileList.GetCurSel();

	static BOOL bFirst;

	if(nContext == MTA_ONBUTTONDOWN)
	{
		if((m_locationData->GetMapTile(tp->xTile, tp->yTile) != bTile) &&
			(m_rcMarquee.IsRectEmpty() || m_rcMarquee.PtInRect(CPoint(tp->xTile, tp->yTile))))
		{
			AddUndoAction(TRUE, tp->xTile, tp->yTile, m_locationData->GetMapTile(tp->xTile, tp->yTile));

			bFirst = TRUE;
			m_locationData->SetMapTile(tp->xTile, tp->yTile, bTile);
			CRect rect(tp->xScreenTile * 16, tp->yScreenTile * 16,
				(tp->xScreenTile + 1) * 16, (tp->yScreenTile + 1) * 16);
			InvalidateRect(&rect);
			bFirst = FALSE;
		}
	}
	else if(nContext == MTA_ONMOUSEMOVE)
	{
		if((m_locationData->GetMapTile(tp->xTile, tp->yTile) != bTile) &&
			(m_rcMarquee.IsRectEmpty() || m_rcMarquee.PtInRect(CPoint(tp->xTile, tp->yTile))))
		{
			AddUndoAction(FALSE, tp->xTile, tp->yTile, m_locationData->GetMapTile(tp->xTile, tp->yTile));

			m_locationData->SetMapTile(tp->xTile, tp->yTile, bTile);
			CRect rect(tp->xScreenTile * 16, tp->yScreenTile * 16,
				(tp->xScreenTile + 1) * 16, (tp->yScreenTile + 1) * 16);
			InvalidateRect(&rect);
		}
	}
	else if(nContext == MTA_MOVENOCAPTURE)
	{
		InvalidateRect(&m_rcTool);

		m_rcToolOld = m_rcTool;

		m_rcTool = CRect(tp->xScreenTile * 16, tp->yScreenTile * 16,
			(tp->xScreenTile + 1) * 16, (tp->yScreenTile + 1) * 16);

		InvalidateRect(&m_rcTool);
	}
	else if(nContext == MTA_DRAWAFTERFLIP)
	{
		if(bFirst)
			tp->pDC->DrawDragRect(&m_rcTool, CSize(2, 2), &m_rcToolOld, CSize(2, 2));
		else
			tp->pDC->DrawDragRect(&m_rcTool, CSize(2, 2), NULL, CSize(2, 2));
	}
}

void CLocationView::DoToolRectFill(int nContext, TOOLPACKAGE* tp)
{
	CFF4Doc* pDoc = GetDocument();
	int bTile = pDoc->m_wndTiles.m_wndTileList.GetCurSel();

	static BOOL bFirst;

	if(nContext == MTA_ONBUTTONDOWN)
	{
		m_rcTool.left = tp->xTile;
		m_rcTool.top = tp->yTile;
		m_rcTool.right = tp->xTile + 1;
		m_rcTool.bottom = tp->yTile + 1;
		m_ptTool = CPoint(tp->xTile, tp->yTile);

		bFirst = TRUE;
		CRect rect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
			(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
		InvalidateRect(&rect);
		bFirst = FALSE;
	}
	else if(nContext == MTA_ONMOUSEMOVE)
	{
		CRect rect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
			(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
		InvalidateRect(&rect);

		m_rcToolOld = m_rcTool;

		m_rcTool.left = min(m_ptTool.x, tp->xTile + 1);
		m_rcTool.top = min(m_ptTool.y, tp->yTile + 1);
		m_rcTool.right = max(m_ptTool.x, tp->xTile + 1);
		m_rcTool.bottom = max(m_ptTool.y, tp->yTile + 1);

		if(m_rcTool.right == m_ptTool.x)
		{
			m_rcTool.left--;
			m_rcTool.right++;
		}
		if(m_rcTool.bottom == m_ptTool.y)
		{
			m_rcTool.top--;
			m_rcTool.bottom++;
		}

		rect = CRect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
			(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
		InvalidateRect(&rect);
	}
	else if(nContext == MTA_ONBUTTONUP)
	{
		CWaitCursor wc;

		BOOL bFirst = TRUE;
		for(int y = min(m_rcTool.top, m_rcTool.bottom); y < max(m_rcTool.top, m_rcTool.bottom); y++)
			for(int x = min(m_rcTool.left, m_rcTool.right); x < max(m_rcTool.left, m_rcTool.right); x++)
			{
				AddUndoAction(bFirst, x, y, m_locationData->GetMapTile(x, y));
				bFirst = FALSE;
				m_locationData->SetMapTile(x, y, bTile);
			}

		m_rcTool.SetRectEmpty();
		Invalidate();
	}
	else if(nContext == MTA_DRAWAFTERFLIP)
	{
		if(!m_rcTool.IsRectEmpty() && GetCapture() == this)
		{
			CRect rect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
					(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
			CRect rectOld((m_rcToolOld.left - GetScrollPosition().x) * 16, (m_rcToolOld.top - GetScrollPosition().y) * 16,
					(m_rcToolOld.right - GetScrollPosition().x) * 16, (m_rcToolOld.bottom - GetScrollPosition().y) * 16);

			if(bFirst)
				tp->pDC->DrawDragRect(&rect, CSize(2, 2), &rectOld, CSize(2, 2));
			else
				tp->pDC->DrawDragRect(&rect, CSize(2, 2), NULL, CSize(2, 2));
		}
	}
	else if(nContext == MTA_ONRBUTTONDOWN)
	{
		if(GetCapture() == this)
		{
			m_rcTool.SetRectEmpty();
			Invalidate();
		}
	}
}

void CLocationView::DoToolMarquee(int nContext, TOOLPACKAGE* tp)
{
	CFF4Doc* pDoc = GetDocument();
	int bTile = pDoc->m_wndTiles.m_wndTileList.GetCurSel();

	static BOOL bFirst;
	static BOOL bValid;

	if(nContext == MTA_ONBUTTONDOWN)
	{
		m_rcTool.left = tp->xTile;
		m_rcTool.top = tp->yTile;
		m_rcTool.right = tp->xTile;
		m_rcTool.bottom = tp->yTile;
		m_ptTool = CPoint(tp->xTile, tp->yTile);

		bValid = FALSE;
		bFirst = TRUE;
		CRect rect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
			(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
		InvalidateRect(&rect);
		bFirst = FALSE;

		if(!m_rcMarquee.IsRectEmpty() && m_pClipboard)
		{
			for(int y = 0; y < m_bClipHeight; y++)
			{
				for(int x = 0; x < m_bClipWidth; x++)
				{
					m_locationData->SetMapTile(x + m_rcMarquee.left, y + m_rcMarquee.top,
						m_pClipboard[y * m_bClipWidth + x]);
				}
			}

			delete m_pClipboard;
			m_pClipboard = NULL;
			m_bClipWidth = m_bClipHeight = 0;
		}
	}
	else if(nContext == MTA_ONMOUSEMOVE)
	{
		bValid = TRUE;

		CRect rect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
			(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
		InvalidateRect(&rect);

		if(!m_bPasting)
		{
			m_rcToolOld = m_rcTool;

			m_rcTool.left = min(m_ptTool.x, tp->xTile + 1);
			m_rcTool.top = min(m_ptTool.y, tp->yTile + 1);
			m_rcTool.right = max(m_ptTool.x, tp->xTile + 1);
			m_rcTool.bottom = max(m_ptTool.y, tp->yTile + 1);

			if(m_rcTool.right == m_ptTool.x)
			{
				m_rcTool.left--;
				m_rcTool.right++;
			}
			if(m_rcTool.bottom == m_ptTool.y)
			{
				m_rcTool.top--;
				m_rcTool.bottom++;
			}
		}

		rect = CRect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
			(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
		InvalidateRect(&rect);
	}
	else if(nContext == MTA_MOVENOCAPTURE)
	{
		if(m_bPasting)
		{
			CRect rect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
				(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
			InvalidateRect(&rect);

			m_rcToolOld = m_rcTool;

			m_rcTool.left = tp->xTile;
			m_rcTool.top = tp->yTile;
			m_rcTool.right = m_rcTool.left + m_bClipWidth;
			m_rcTool.bottom = m_rcTool.top + m_bClipHeight;

			rect = CRect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
				(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
			InvalidateRect(&rect);
		}
	}
	else if(nContext == MTA_ONBUTTONUP)
	{
		if(bValid)
		{
			// If it's valid, we just -completed- a marquee.  Cut the selection inside.
			m_rcMarquee = m_rcTool;

			m_bClipWidth = m_rcMarquee.Width();
			m_bClipHeight = m_rcMarquee.Height();

			if(m_pClipboard) delete m_pClipboard;
			m_pClipboard = new BYTE[m_bClipWidth * m_bClipHeight];

			for(int y = 0; y < m_bClipHeight; y++)
			{
				for(int x = 0; x < m_bClipWidth; x++)
				{
					m_pClipboard[y * m_bClipWidth + x] =
						m_locationData->GetMapTile(x + m_rcMarquee.left, y + m_rcMarquee.top);
					m_locationData->SetMapTile(x + m_rcMarquee.left, y + m_rcMarquee.top, -1);
				}
			}
		}
		else
		{
			// If it's valid, we just -removed- a marquee.  Paste the selection in the
			// destination.
			if(!m_rcMarquee.IsRectEmpty() && m_pClipboard)
			{
				for(int y = 0; y < m_bClipHeight; y++)
				{
					for(int x = 0; x < m_bClipWidth; x++)
					{
						m_locationData->SetMapTile(x + m_rcMarquee.left, y + m_rcMarquee.top,
							m_pClipboard[y * m_bClipWidth + x]);
					}
				}

				delete m_pClipboard;
				m_pClipboard = NULL;
				m_bClipWidth = m_bClipHeight = 0;
			}

			m_rcMarquee.SetRectEmpty();
		}

		m_rcTool.SetRectEmpty();
		m_ptMarqueeDrag = CPoint(-1, -1);
		Invalidate();
	}
	else if(nContext == MTA_DRAWAFTERFLIP)
	{
		if(!m_rcTool.IsRectEmpty() && (GetCapture() == this || m_bPasting))
		{
			CRect rect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
					(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
			CRect rectOld((m_rcToolOld.left - GetScrollPosition().x) * 16, (m_rcToolOld.top - GetScrollPosition().y) * 16 + 2,
					(m_rcToolOld.right - GetScrollPosition().x) * 16, (m_rcToolOld.bottom - GetScrollPosition().y) * 16 + 2);

			if(bFirst)
				tp->pDC->DrawDragRect(&rect, CSize(2, 2), &rectOld, CSize(2, 2));
			else
				tp->pDC->DrawDragRect(&rect, CSize(2, 2), NULL, CSize(2, 2));
		}
	}
	else if(nContext == MTA_ONRBUTTONDOWN)
	{
		if(GetCapture() == this)
		{
			m_rcTool.SetRectEmpty();
			bValid = FALSE;
			Invalidate();
		}
	}
}

void CLocationView::DoToolPickup(int nContext, TOOLPACKAGE* tp)
{
	CFF4Doc* pDoc = GetDocument();
	int bTile = pDoc->m_wndTiles.m_wndTileList.GetCurSel();
	static BOOL bFirst = TRUE;

	if(nContext == MTA_ONBUTTONDOWN || nContext == MTA_ONMOUSEMOVE)
	{
		bTile = m_locationData->GetMapTile(tp->xTile, tp->yTile);
		pDoc->m_wndTiles.m_wndTileList.SetCurSel(bTile);
	}
	else if(nContext == MTA_MOVENOCAPTURE)
	{
		InvalidateRect(&m_rcTool);
		bFirst = FALSE;

		m_rcToolOld = m_rcTool;

		m_rcTool = CRect(tp->xScreenTile * 16, tp->yScreenTile * 16,
			(tp->xScreenTile + 1) * 16, (tp->yScreenTile + 1) * 16);

		InvalidateRect(&m_rcTool);
	}
	else if(nContext == MTA_DRAWAFTERFLIP)
	{
		if(bFirst)
			tp->pDC->DrawDragRect(&m_rcTool, CSize(2, 2), &m_rcToolOld, CSize(2, 2));
		else
			tp->pDC->DrawDragRect(&m_rcTool, CSize(2, 2), NULL, CSize(2, 2));
	}
}

void CLocationView::DoToolPointer(int nContext, TOOLPACKAGE* tp)
{
	CFF4Doc* pDoc = GetDocument();
	int bTile = pDoc->m_wndTiles.m_wndTileList.GetCurSel();
	static BOOL bFirst = TRUE;

	if(nContext == MTA_ONBUTTONDOWN || nContext == MTA_ONMOUSEMOVE)
	{
		int nOldSel = pDoc->m_wndTriggers.m_wndTriggerList.GetCurSel();

		Entrance* pEntr = m_locationData->FindEntranceFromSource(tp->xTile, tp->yTile);

		for(int i = 0; i < m_locationData->entrances.GetSize(); i++)
		{
			if(pEntr == m_locationData->entrances[i] && nOldSel != i)
			{
				pDoc->m_wndTriggers.m_wndTriggerList.SetCurSel(i);
				pDoc->m_wndTriggers.SendCommandMessage(&pDoc->m_wndTriggers.m_wndTriggerList, LBN_SELCHANGE);

				break;
			}
		}
	}
	else if(nContext == MTA_MOVENOCAPTURE)
	{
		InvalidateRect(&m_rcTool);
		bFirst = FALSE;

		m_rcToolOld = m_rcTool;

		m_rcTool = CRect(tp->xScreenTile * 16, tp->yScreenTile * 16,
			(tp->xScreenTile + 1) * 16, (tp->yScreenTile + 1) * 16);

		InvalidateRect(&m_rcTool);
	}
	else if(nContext == MTA_DRAWAFTERFLIP)
	{
		if(bFirst)
			tp->pDC->DrawDragRect(&m_rcTool, CSize(2, 2), &m_rcToolOld, CSize(2, 2));
		else
			tp->pDC->DrawDragRect(&m_rcTool, CSize(2, 2), NULL, CSize(2, 2));
	}
}

void CLocationView::AddUndoAction(BOOL bFirst, int x, int y, BYTE bPrevTile)
{
	UNDOACTION ua;
	ua.bFirstAction = bFirst;
	ua.x = x;
	ua.y = y;
	ua.bPrevTile = bPrevTile;

	if(GetUndoActionCount() == 200)
	{
		m_undoActions.RemoveAt(0);
	}

	m_undoActions.Add(ua);
}

void CLocationView::OnTimer(UINT nIDEvent) 
{
	if(nIDEvent == 31337)
	{
		CPoint point;  GetCursorPos(&point);  ScreenToClient(&point);
		CRect clientRect;  GetClientRect(&clientRect);

		ScrollToPosition(GetScrollPosition() + m_ptScrollOffset);

		TOOLPACKAGE tp;
		tp.xTile = GetScrollPosition().x + point.x / 16;
		if(tp.xTile < 0) tp.xTile = 0;
		else if(tp.xTile > m_locationData->gLocationData->mapSize.cx - 1)
			tp.xTile = m_locationData->gLocationData->mapSize.cx - 1;
		tp.yTile = GetScrollPosition().y + point.y / 16;
		if(tp.yTile < 0) tp.yTile = 0;
		else if(tp.yTile > m_locationData->gLocationData->mapSize.cy - 1)
			tp.yTile = m_locationData->gLocationData->mapSize.cy - 1;
		tp.xScreenTile = point.x / 16;
		if(tp.xScreenTile < 0) tp.xScreenTile = 0;
		else if(tp.xScreenTile > clientRect.Width() / 16) tp.xScreenTile = clientRect.Width() / 16;
		tp.yScreenTile = point.y / 16;
		if(tp.yScreenTile < 0) tp.yScreenTile = 0;
		else if(tp.yScreenTile > clientRect.Height() / 16) tp.yScreenTile = clientRect.Height() / 16;

		DoTool(MTA_ONMOUSEMOVE, &tp);
	}
	else
		CDirectDrawView::OnTimer(nIDEvent);
}

void CLocationView::OnChangeTool(UINT nID)
{
	ASSERT(nID >= ID_TOOLS_POINTER && nID <= ID_TOOLS_FILL);

	int nOldTool = m_nCurrentTool;
	m_nCurrentTool = nID;

	if(nOldTool == ID_TOOLS_POINTER || m_nCurrentTool == ID_TOOLS_POINTER)
		Invalidate();
}

void CLocationView::OnUpdateTool(CCmdUI* pCmdUI)
{
	if(pCmdUI->m_nID == m_nCurrentTool)
	{
		pCmdUI->SetCheck(TRUE);
		return;
	}

	// default to not checked
	pCmdUI->SetCheck(FALSE);

	if(pCmdUI->m_nID == ID_TOOLS_FILL || pCmdUI->m_nID == ID_TOOLS_LINE || pCmdUI->m_nID == ID_TOOLS_RECT)
		pCmdUI->Enable(FALSE);
}

void CLocationView::OnEditUndo() 
{
	BOOL bDone = FALSE;

	while(!bDone)
	{
		UNDOACTION ua = m_undoActions[m_undoActions.GetUpperBound()];

		bDone = ua.bFirstAction;

		m_locationData->SetMapTile(ua.x, ua.y, ua.bPrevTile);

		if(IsTileVisible(ua.x, ua.y))
		{
			CRect tileRect = GetTileScreenRect(ua.x, ua.y);
			InvalidateRect(&tileRect);
		}

		m_undoActions.RemoveAt(m_undoActions.GetUpperBound());
		
		if(m_undoActions.GetSize() == 0) bDone = TRUE;
	}
}

void CLocationView::OnUpdateEditUndo(CCmdUI* pCmdUI) 
{
	if(m_undoActions.GetSize())
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

CRect CLocationView::GetVisibleTileBounds()
{
	CRect bounds;

	CPoint ptScroll = GetScrollPosition();
	CRect rect;  GetClientRect(&rect);

	bounds.left = ptScroll.x;
	bounds.top = ptScroll.y;
	bounds.right = ptScroll.x + rect.Width() / 16;
	bounds.bottom = ptScroll.y + rect.Height() / 16;

	return bounds;
}

CRect CLocationView::GetTileScreenRect(int x, int y)
{
	CPoint ptScroll = GetScrollPosition();

	CRect screenRect((x - ptScroll.x) * 16, (y - ptScroll.y) * 16,
		(x - ptScroll.x + 1) * 16, (y - ptScroll.y + 1) * 16);

	return screenRect;
}

BOOL CLocationView::IsTileVisible(int x, int y)
{
	CRect bounds = GetVisibleTileBounds();

	if((x >= bounds.left && x <= bounds.right) && (y >= bounds.top && y <= bounds.bottom))
		return TRUE;
	
	return FALSE;
}

int CLocationView::GetUndoActionCount()
{
	int nActions = 0;

	for(int i = 0; i < m_undoActions.GetSize(); i++)
	{
		UNDOACTION ua = m_undoActions[i];
		if(ua.bFirstAction) nActions++;
	}

	return nActions;
}

void CLocationView::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	TOOLPACKAGE tp;
	tp.xTile = GetScrollPosition().x + point.x / 16;
	tp.yTile = GetScrollPosition().y + point.y / 16;
	tp.xScreenTile = point.x / 16;
	tp.yScreenTile = point.y / 16;

	DoTool(MTA_ONCONTEXTMENU, &tp);
	ReleaseCapture();
}

void CLocationView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	TOOLPACKAGE tp;
	tp.xTile = GetScrollPosition().x + point.x / 16;
	tp.yTile = GetScrollPosition().y + point.y / 16;
	tp.xScreenTile = point.x / 16;
	tp.yScreenTile = point.y / 16;

	DoTool(MTA_ONRBUTTONDOWN, &tp);
	ReleaseCapture();
	
	CDirectDrawView::OnRButtonDown(nFlags, point);
}

LRESULT CLocationView::OnMouseLeave(WPARAM wParam, LPARAM lParam)
{
	CClientDC dc(this);
	if(!m_rcTool.IsRectEmpty())
	{
		dc.DrawDragRect(CRect(0, 0, 0, 0), CSize(2, 2), &m_rcTool, CSize(2, 2));
		m_rcTool.SetRectEmpty();
	}

	return 0;
}

void CLocationView::OnUpdateEditCutOrCopy(CCmdUI* pCmdUI) 
{
	if(m_rcMarquee.IsRectEmpty()) pCmdUI->Enable(FALSE);
	else pCmdUI->Enable(TRUE);
}

void CLocationView::OnUpdateEditPaste(CCmdUI* pCmdUI) 
{
	COleDataObject obj;

	if(obj.AttachClipboard())
	{
		if(obj.IsDataAvailable(App.clipboardWorldMap))
		{
			pCmdUI->Enable(TRUE);
			if(m_bPasting) pCmdUI->SetCheck(TRUE);
			else pCmdUI->SetCheck(FALSE);
			return;
		}
	}

	pCmdUI->SetCheck(FALSE);
	pCmdUI->Enable(FALSE);
}

void CLocationView::OnEditCut() 
{
	COleDataSource*	pSource = new COleDataSource();
	CSharedFile	sf(GMEM_MOVEABLE | GMEM_DDESHARE | GMEM_ZEROINIT);

	sf.Write(&m_bClipWidth, 1);
	sf.Write(&m_bClipHeight, 1);
	sf.Write(m_pClipboard, m_bClipWidth * m_bClipHeight);

	delete m_pClipboard;
	m_pClipboard = NULL;
	m_bClipWidth = m_bClipHeight = 0;
	Invalidate();

	HGLOBAL hMem = sf.Detach();
	if(!hMem) return;
	
	pSource->CacheGlobalData(App.clipboardWorldMap, hMem);
	pSource->SetClipboard();

//#if _MFC_VER <= 0x0600
//	::GlobalUnlock(hMem);
//#endif 
}

void CLocationView::OnEditCopy() 
{
	COleDataSource*	pSource = new COleDataSource();
	CSharedFile	sf(GMEM_MOVEABLE | GMEM_DDESHARE | GMEM_ZEROINIT);

	for(int y = 0; y < m_bClipHeight; y++)
	{
		for(int x = 0; x < m_bClipWidth; x++)
		{
			m_locationData->SetMapTile(x + m_rcMarquee.left, y + m_rcMarquee.top,
				m_pClipboard[y * m_bClipWidth + x]);
		}
	}

//	delete m_pClipboard;
//	m_pClipboard = NULL;
//	m_bClipWidth = m_bClipHeight = 0;

	sf.Write(&m_bClipWidth, 1);
	sf.Write(&m_bClipHeight, 1);
	sf.Write(m_pClipboard, m_bClipWidth * m_bClipHeight);

	HGLOBAL hMem = sf.Detach();
	if(!hMem) return;
	
	pSource->CacheGlobalData(App.clipboardWorldMap, hMem);
	pSource->SetClipboard();

#if _MFC_VER <= 0x0600
	::GlobalUnlock(hMem);
#endif 
}

void CLocationView::OnEditPaste() 
{
	COleDataObject obj;

	if(obj.AttachClipboard())
	{
		if(obj.IsDataAvailable(App.clipboardWorldMap))
		{
			m_bPasting = TRUE;
		}
	}
}

BOOL CLocationView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	CPoint point;  ::GetCursorPos(&point);  ScreenToClient(&point);
	int nHit = m_rcTracker.HitTest(point);

	if(nHit == CRectTracker::hitMiddle && m_nCurrentTool == ID_TOOLS_MARQUEE)
	{
//		::SetCursor(AfxGetApp()->LoadStandardCursor(IDC_SIZEALL));

		if(m_ptMarqueeDrag == CPoint(-1, -1))
		{
			m_ptMarqueeDrag.x = (point.x - m_rcTracker.m_rect.left) / 16;
			m_ptMarqueeDrag.y = (point.y - m_rcTracker.m_rect.top) / 16;
		}
		return TRUE;
	}
	else
	{
		// Change the cursor based on which tool is selected.
		int nCursor = 0;

		if(m_nCurrentTool == ID_TOOLS_PENCIL) nCursor = IDC_TOOLS_PENCIL;
		else if(m_nCurrentTool == ID_TOOLS_PICKUP) nCursor = IDC_TOOLS_PICKUP;
		else if(m_nCurrentTool == ID_TOOLS_MARQUEE) nCursor = IDC_TOOLS_MARQUEE;
		else if(m_nCurrentTool == ID_TOOLS_RECT || m_nCurrentTool == ID_TOOLS_RECTFILL)
			nCursor = IDC_TOOLS_RECT;
		else if(m_nCurrentTool == ID_TOOLS_FILL) nCursor = IDC_TOOLS_FILL;
		else if(m_nCurrentTool == ID_TOOLS_LINE) nCursor = IDC_TOOLS_LINE;

		if(nCursor && nHitTest == HTCLIENT)
		{
			::SetCursor(AfxGetApp()->LoadCursor(MAKEINTRESOURCE(nCursor)));
			return TRUE;
		}
		else
			return CDirectDrawView::OnSetCursor(pWnd, nHitTest, message);
	}
}
