// LocationView.h : interface of the CLocationView class
//
/////////////////////////////////////////////////////////////////////////////

#include "DirectDrawView.h"

class CLocationView : public CDirectDrawView
{
protected: // create from serialization only
	CLocationView();
	DECLARE_DYNCREATE(CLocationView)

	typedef struct __TOOLPACKAGE
	{
		int xTile;
		int yTile;
		int xScreenTile;
		int yScreenTile;
		LPDDSURFACEDESC2 lpDest;
		CDC* pDC;
	} TOOLPACKAGE;

	typedef struct __UNDOACTION
	{
		BOOL bFirstAction;
		int x;
		int y;
		BYTE bPrevTile;
	} UNDOACTION;

	enum ToolContexts
	{
		MTA_ONBUTTONDOWN = 1,
		MTA_ONMOUSEMOVE,
		MTA_ONBUTTONUP,
		MTA_DRAW,
		MTA_DRAWNOLOCK,
		MTA_SETCURSOR,
		MTA_MOVENOCAPTURE,
		MTA_ONDOUBLECLICK,
		MTA_ONCONTEXTMENU,
		MTA_ONRBUTTONDOWN,
		MTA_DRAWAFTERFLIP
	};

	enum OverScrollDirections
	{
		OSD_LEFT = 1,
		OSD_RIGHT,
		OSD_UP,
		OSD_DOWN
	};

// Attributes
public:
	CFF4Doc* GetDocument();

	LocationData* m_locationData;

protected:
	UINT m_nCurrentTool;

	CPoint m_ptCurrentTrigger;

	CPoint m_ptMouseTile;
	CPoint m_ptScrollOffset;
	CPoint m_ptLastMouse;

	CRect m_rcTool, m_rcToolOld;
	CPoint m_ptTool;
	CPoint m_ptMarqueeDrag;

	CRect m_rcMarquee;
	CRectTracker m_rcTracker;

	CArray<UNDOACTION, UNDOACTION> m_undoActions;
	CPoint m_fillSeed;
	BYTE m_seedTile;

	BOOL m_bTimerSet;

	BYTE m_bClipWidth, m_bClipHeight;
	BYTE* m_pClipboard;
	BOOL m_bPasting;

// Operations
public:

protected:
	void Put8x8x3Tile(DDSURFACEDESC2* ddsd, int dx, int dy, LocationTilePart* pTilePart);
	void Put8x8x4Tile(DDSURFACEDESC2* ddsd, int dx, int dy, LocationTilePart* pTilePart);
	void Put16x16Tile(DDSURFACEDESC2* ddsd, int dx, int dy, BYTE bTile);

	void DoTool(int nContext, TOOLPACKAGE* tp);
	void DoToolPencil(int nContext, TOOLPACKAGE* tp);
	void DoToolPickup(int nContext, TOOLPACKAGE* tp);
	void DoToolMarquee(int nContext, TOOLPACKAGE* tp);
	void DoToolPointer(int nContext, TOOLPACKAGE* tp);
	void DoToolRectFill(int nContext, TOOLPACKAGE* tp);
//	void DoToolFill(int nContext, TOOLPACKAGE* tp);

	void AddUndoAction(BOOL bFirst, int x, int y, BYTE bPrevTile);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CLocationView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnInitialUpdate();
	virtual BOOL OnScroll(UINT nScrollCode, UINT nPos, BOOL bDoScroll = TRUE);
	protected:
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
	//}}AFX_VIRTUAL

// Implementation
public:
	CRect GetTileScreenRect(int x, int y);
	void SetCurrentTrigger(CPoint pt);
	virtual ~CLocationView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
	int GetUndoActionCount();
	BOOL IsTileVisible(int x, int y);
	CRect GetVisibleTileBounds();

// Generated message map functions
protected:
	//{{AFX_MSG(CLocationView)
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnEditUndo();
	afx_msg void OnUpdateEditUndo(CCmdUI* pCmdUI);
	afx_msg void OnViewGridlines();
	afx_msg void OnUpdateViewGridlines(CCmdUI* pCmdUI);
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnUpdateEditPaste(CCmdUI* pCmdUI);
	afx_msg void OnEditCut();
	afx_msg void OnEditCopy();
	afx_msg void OnEditPaste();
	afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
	//}}AFX_MSG
	afx_msg void OnChangeTool(UINT nID);
	afx_msg void OnUpdateTool(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditCutOrCopy(CCmdUI* pCmdUI);
	LRESULT OnMouseLeave(WPARAM wParam, LPARAM lParam);
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in LocationView.cpp
inline CFF4Doc* CLocationView::GetDocument()
   { return (CFF4Doc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////
