// MainFrame.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "FF4Edit.h"

#include "MainFrame.h"

#include "FF4Doc.h"
#include "FF4TreeView.h"
#include "MapView.h"
#include "EditorInfoView.h"

#include <afxole.h>
#include <afxpriv.h>

#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_UPDATEFLEXPANE, OnUpdateFlexPane)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_pBand = NULL;
	m_bFirstTime = TRUE;
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.CreateEx(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	if (!m_wndReBar.Create(this) ||
		!m_wndReBar.AddBar(&m_wndToolBar))
	{
		TRACE0("Failed to create rebar\n");
		return -1;      // fail to create
	}

	// TODO: Remove this if you don't want tool tips or a resizeable toolbar
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	// Create the primary surface.  We do it here because in a windowed application, one
	// primary surface covers all our windows--the primary surface is just a DirectDraw-ized
	// pointer to the GDI for the entire screen.
	DDSURFACEDESC2 ddsd;

	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_CAPS;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

	HRESULT rval = DDraw.lpDD->CreateSurface(&ddsd, &DDraw.lpSurfFront, NULL);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	DDraw.ddpfPrimary.dwSize = sizeof(DDPIXELFORMAT);
	rval = DDraw.lpSurfFront->GetPixelFormat(&DDraw.ddpfPrimary);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	// Check for a proper screenmode (if running in 256 colors, quit).
	if(DDraw.ddpfPrimary.dwRGBBitCount < 16)
	{
		AfxMessageBox("This program cannot currently be run in\na 256-color screenmode.  Please increase the\nnumber of colors to at 16-bit or higher.", MB_OK | MB_ICONSTOP);
		SendMessage(WM_CLOSE);
	}

	rval = DDraw.lpDD->CreateClipper(0, &DDraw.lpClipper, NULL);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	rval = DDraw.lpSurfFront->SetClipper(DDraw.lpClipper);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	return 0;
}

void CMainFrame::OnDestroy() 
{
	RELEASE(DDraw.lpClipper);
	RELEASE(DDraw.lpSurfFront);

	CFrameWnd::OnDestroy();
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext) 
{
	// Set the cooperative level here--after the window was created, but before it's
	// displayed.
	HRESULT rval = DDraw.lpDD->SetCooperativeLevel(GetSafeHwnd(), DDSCL_NORMAL);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	if(!m_wndSplitter.CreateStatic(this, 1, 2))
	{
		return FALSE;
	}

	if(!m_wndSplitter.CreateView(0, 0, RUNTIME_CLASS(CFF4TreeView), CSize(180, 0), pContext) ||
		!m_wndSplitter.CreateView(0, 1, RUNTIME_CLASS(CEditorInfoView), CSize(0, 0), pContext))
	{
		m_wndSplitter.DestroyWindow();
		return FALSE;
	}

	return TRUE;
}

LONG CMainFrame::OnUpdateFlexPane(UINT pFP, LONG /* unused */)
{
	FLEXPANE* pFlex = (FLEXPANE*)pFP;

	// If the panes are the same and of class CEditorInfoView, make no change.
	if(m_wndSplitter.GetPane(0, 1)->IsKindOf(RUNTIME_CLASS(CEditorInfoView)) &&
		(lstrcmp(pFlex->pClass->m_lpszClassName, _T("CEditorInfoView")) == 0))
		return NULL;

	CRect rect;  GetClientRect(&rect);
	int cxCur, cyCur, cxMin, cyMin;
	m_wndSplitter.GetColumnInfo(1, cxCur, cxMin);
	m_wndSplitter.GetRowInfo(0, cyCur, cyMin);
	CSize paneSize(cxCur, cyCur);
	
	CCreateContext context;
	context.m_pNewViewClass = pFlex->pClass;
	context.m_pCurrentDoc = GetActiveDocument();
	context.m_pCurrentFrame = this;
	context.m_pNewDocTemplate = context.m_pCurrentDoc->GetDocTemplate();
	context.m_pLastView = (CView*)m_wndSplitter.GetPane(0, 0);
	
	// Called in lieu of CWndSplitter::DeleteView() because it causes an assertion
	// if the pane isn't derived from CView (plain stupid, considering all the other
	// CSplitterWnd functions deal with CWnd and not CView).  This code was just ripped
	// from the MFC source for the function and the ASSERT was removed.
	CWnd* pPane = m_wndSplitter.GetPane(0, 1);
	CRuntimeClass* pOldClass = pPane->GetRuntimeClass();

	if(m_wndSplitter.GetActivePane() == pPane) m_wndSplitter.ActivateNext(FALSE);
	pPane->DestroyWindow();
	
	m_wndSplitter.CreateView(0, 1, pFlex->pClass, paneSize, &context);
	CWnd* pWnd = m_wndSplitter.GetPane(0, 1);

	if(pFlex->nID)
		m_wndToolBar.LoadToolBar(pFlex->nID);
	else
		m_wndToolBar.LoadToolBar(IDR_MAINFRAME);

	RecalcLayout();
	m_wndSplitter.RecalcLayout();

	CFF4Doc* pDoc = (CFF4Doc*)((CView*)pWnd)->GetDocument();

	// If the view classes are different, create new dialogs.
	if(lstrcmp(pFlex->pClass->m_lpszClassName, pOldClass->m_lpszClassName) != 0)
	{
		pDoc->DestroyDialogs(pOldClass);
		pDoc->CreateDialogs(pFlex->pClass, (CView*)pWnd);
	}
	else
	{
		pDoc->UpdateDialogs(pFlex->pClass, (CView*)pWnd);
	}

	if(pWnd->IsKindOf(RUNTIME_CLASS(CView)))
	{
		pWnd->SendMessage(WM_INITIALUPDATE, 0, 0);
		pDoc->UpdateAllViews((CView*)m_wndSplitter.GetPane(0, 0), pFlex->dwData, NULL);
	}

	return NULL;
}

void CMainFrame::ActivateFrame(int nCmdShow) 
{
	if(m_bFirstTime)
	{
		m_bFirstTime = FALSE;

		WINDOWPLACEMENT wp;
		UINT flags;

		if(Settings.wsMainWindow.bMinimized)
		{
			nCmdShow = SW_SHOWMINNOACTIVE;
			if(Settings.wsMainWindow.bMaximized)
			{
				flags = WPF_RESTORETOMAXIMIZED;
			}
		}
		else
		{
			if(Settings.wsMainWindow.bMaximized)
			{
				nCmdShow = SW_SHOWMAXIMIZED;
				flags = WPF_RESTORETOMAXIMIZED;
			}
			else
			{
				nCmdShow = SW_SHOWNORMAL;
				flags = 0;
			}
		}

		wp.length = sizeof(WINDOWPLACEMENT);
		wp.showCmd = nCmdShow;
		wp.flags = flags;
		wp.ptMinPosition = CPoint(0, 0);
		wp.ptMaxPosition = CPoint(-::GetSystemMetrics(SM_CXBORDER), -::GetSystemMetrics(SM_CYBORDER));
		wp.rcNormalPosition = Settings.wsMainWindow.rect;

		SetWindowPlacement(&wp);
	}
	
	CFrameWnd::ActivateFrame(nCmdShow);
}

void CMainFrame::OnClose() 
{
	WINDOWPLACEMENT wp;
	wp.length = sizeof(WINDOWPLACEMENT);

	GetWindowPlacement(&wp);

	if(wp.showCmd == SW_SHOWNORMAL)
	{
		Settings.wsMainWindow.bMinimized = FALSE;
		Settings.wsMainWindow.bMaximized = FALSE;
	}
	else if(wp.showCmd == SW_SHOWMAXIMIZED)
	{
		Settings.wsMainWindow.bMinimized = FALSE;
		Settings.wsMainWindow.bMaximized = TRUE;
	}
	else if(wp.showCmd == SW_SHOWMINIMIZED)
	{
		Settings.wsMainWindow.bMinimized = TRUE;
		
		if(wp.flags & WPF_RESTORETOMAXIMIZED)
			Settings.wsMainWindow.bMaximized = TRUE;
		else
			Settings.wsMainWindow.bMaximized = FALSE;
	}

	Settings.wsMainWindow.rect = wp.rcNormalPosition;

	CFrameWnd::OnClose();
}
