// MapObjects.cpp : implementation of the CFF4Doc class helpers
//

#include "stdafx.h"
#include "FF4Edit.h"
#include "FF4Doc.h"

/////////////////////////////////////////////////////////////////////

WorldData::WorldData()
{
	objectType = ObjWorldData;

	map = NULL;
	tiles = NULL;

	ppdLastScrollPos = CPoint(0, 0);
}

WorldData::~WorldData()
{
	for(int i = 0; i < entrances.GetSize(); i++)
		delete entrances[i];

	if(map) delete map;
	if(tiles) delete tiles;
}

BOOL WorldData::ReadMapData(int nMap)
{
	if(nMap == MAP_OVERWORLD)
	{
		size = CSize(256, 256);
		map = new BYTE[256 * 256];

		WORD wRows[256];
		pDoc->m_pFile->Seek(0xB0200, CFile::begin);
		pDoc->m_pFile->Read(&wRows, 512);

		DWORD dwOffset = 0;
		pDoc->m_pFile->Seek(0x7CFB, CFile::begin);
		pDoc->m_pFile->Read(&dwOffset, 3);
		dwOffset = LoROMToSMC(dwOffset);

		// Seek to the beginning of the actual overworld map rows.
		for(int i = 0; i < 256; i++)
		{
			pDoc->m_pFile->Seek(dwOffset + wRows[i], CFile::begin);

			BOOL bEndRow = FALSE;
			int xPos = 0;

			while(!bEndRow)
			{
				BYTE bTile = 0;
				pDoc->m_pFile->Read(&bTile, 1);

				if(bTile == 0xFF)
				{
					bEndRow = TRUE;
					break;
				}
				else if(bTile == 0x00)
				{
					map[i * 256 + xPos++] = 0x00;
					map[i * 256 + xPos++] = 0x70;
					map[i * 256 + xPos++] = 0x71;
					map[i * 256 + xPos++] = 0x72;
				}
				else if(bTile == 0x10)
				{
					map[i * 256 + xPos++] = 0x10;
					map[i * 256 + xPos++] = 0x73;
					map[i * 256 + xPos++] = 0x74;
					map[i * 256 + xPos++] = 0x75;
				}
				else if(bTile == 0x20)
				{
					map[i * 256 + xPos++] = 0x20;
					map[i * 256 + xPos++] = 0x76;
					map[i * 256 + xPos++] = 0x77;
					map[i * 256 + xPos++] = 0x78;
				}
				else if(bTile == 0x30)
				{
					map[i * 256 + xPos++] = 0x30;
					map[i * 256 + xPos++] = 0x79;
					map[i * 256 + xPos++] = 0x7A;
					map[i * 256 + xPos++] = 0x7B;
				}
				else if(bTile >= 0x80)
				{
					BYTE nCount = 0;
					pDoc->m_pFile->Read(&nCount, 1);
					nCount++;
					if(nCount == 0) nCount = 255;

					for(int j = 0; j < nCount; j++, xPos++)
						map[i * 256 + xPos] = bTile - 0x80;

					if(nCount == 255) map[i * 256 + 255] = bTile - 0x80;
				}
				else
				{
					map[i * 256 + xPos++] = bTile;
				}
			}
		}
	}
	else if(nMap == MAP_UNDERGROUND)
	{
		size = CSize(256, 256);
		map = new BYTE[256 * 256];

		WORD wRows[256];
		pDoc->m_pFile->Seek(0xB0400, CFile::begin);
		pDoc->m_pFile->Read(&wRows, 512);

		DWORD dwOffset = 0;
		pDoc->m_pFile->Seek(0x7D89, CFile::begin);
		pDoc->m_pFile->Read(&dwOffset, 3);
		dwOffset = LoROMToSMC(dwOffset);

		// Seek to the beginning of the actual overworld map rows.
		for(int i = 0; i < 256; i++)
		{
			pDoc->m_pFile->Seek(dwOffset + wRows[i], CFile::begin);

			BOOL bEndRow = FALSE;
			int xPos = 0;

			while(!bEndRow)
			{
				BYTE bTile = 0;
				pDoc->m_pFile->Read(&bTile, 1);

				if(bTile == 0xFF)
				{
					bEndRow = TRUE;
					break;
				}
				else if(bTile >= 0x80)
				{
					BYTE nCount = 0;
					pDoc->m_pFile->Read(&nCount, 1);
					nCount++;
					if(nCount == 0) nCount = 255;

					for(int j = 0; j < nCount; j++, xPos++)
						map[i * 256 + xPos] = bTile - 0x80;

					if(nCount == 255) map[i * 256 + 255] = bTile - 0x80;
				}
				else
				{
					map[i * 256 + xPos++] = bTile;
				}
			}
		}
	}
	else if(nMap == MAP_MOON)
	{
		size = CSize(64, 64);
		map = new BYTE[64 * 64];

		WORD wRows[64];
		pDoc->m_pFile->Seek(0xB0600, CFile::begin);
		pDoc->m_pFile->Read(&wRows, 128);

		DWORD dwOffset = 0;
		pDoc->m_pFile->Seek(0x7DD6, CFile::begin);
		pDoc->m_pFile->Read(&dwOffset, 3);
		dwOffset = LoROMToSMC(dwOffset);

		// Seek to the beginning of the actual overworld map rows.
		for(int i = 0; i < 64; i++)
		{
			pDoc->m_pFile->Seek(dwOffset + wRows[i], CFile::begin);

			BOOL bEndRow = FALSE;
			int xPos = 0;

			while(!bEndRow)
			{
				BYTE bTile = 0;
				pDoc->m_pFile->Read(&bTile, 1);

				if(bTile == 0xFF)
				{
					bEndRow = TRUE;
					break;
				}
				else if(bTile >= 0x80)
				{
					BYTE nCount = 0;
					pDoc->m_pFile->Read(&nCount, 1);
					nCount++;

					for(int j = 0; j < nCount; j++, xPos++)
						map[i * 64 + xPos] = bTile - 0x80;
				}
				else
				{
					map[i * 64 + xPos++] = bTile;
				}
			}
		}
	}

	return TRUE;
}

BOOL WorldData::ReadTileData(DWORD dwOffset)
{
	// Extract the tiles (as an array of BYTEs) from the ROM.  They are stored
	// in a chunky sequential 4 bits per pixel format.  The tile data is 8192 
	// bytes long.

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	tiles = new BYTE[8192];
	pDoc->m_pFile->Read(tiles, 8192);

	return TRUE;
}

BOOL WorldData::ReadEnemyLayout(DWORD dwOffset, DWORD dwLength)
{
	// Extract the tiles (as an array of BYTEs) from the ROM.  They are stored
	// in a chunky sequential 4 bits per pixel format.  The tile data is 8192 
	// bytes long.

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	pDoc->m_pFile->Read(enemyLayout, dwLength);

	return TRUE;
}

BOOL WorldData::ReadEnemyOdds(DWORD dwOffset, DWORD dwLength)
{
	// Extract the tiles (as an array of BYTEs) from the ROM.  They are stored
	// in a chunky sequential 4 bits per pixel format.  The tile data is 8192 
	// bytes long.

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	pDoc->m_pFile->Read(enemyOdds, dwLength);

	return TRUE;
}

BOOL WorldData::ReadTileFormations(DWORD dwOffset)
{
	// Extract the information telling the game how to combine the various
	// 8x8 tiles to form 16x16 tiles used on the map.  This is useful because
	// it allows identical 8x8 tiles to be reused in different 16x16 tiles,
	// conserving memory.  There are 128 16x16 tiles formed from 4 8x8 tiles,
	// making a total of 512 bytes.

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	pDoc->m_pFile->Read(&tileFormations, 512);

	return TRUE;
}

BOOL WorldData::ReadTileProperties(DWORD dwOffset)
{
	// Extract the information assigning various properties for each of the
	// 128 16x16 tiles.  This information tells the game engine which tiles
	// can be walked on, whether enemy encounters are activated, among other
	// things.  Each property is represented by a WORD, totalling 256 bytes
	// of data for 128 tiles.

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	pDoc->m_pFile->Read(&tileProperties, 256);

	return TRUE;
}

BOOL WorldData::ReadTilePalettes(DWORD dwOffset)
{
	// Extract the data associating each 8x8 tile with one of the 16 color
	// palettes.  Each entry is one byte, for each of the 256 tiles, so
	// there are naturally 256 bytes of data.

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	pDoc->m_pFile->Read(&tilePalettes, 256);

	return TRUE;
}

BOOL WorldData::ReadPalettes(DWORD dwOffset)
{
	// Extract the actual color information from the ROM.  There are 64 colors
	// per map, each color being represented by a WORD in a 555 BGR format.
	// This makes a total of 128 bytes per map.

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	pDoc->m_pFile->Read(&palette, 128);

	return TRUE;
}

BOOL WorldData::ReadEntrances(DWORD dwOffset)
{
	// Create a new instance of an Entrance class and fill it with the
	// necessary parameters for every entrance on the map (each is 5 bytes).
	
	DWORD dwActualOffset = 0, dwLength = 0;
	WORD wEntranceOffset = 0, wNextOffset = 0;

	pDoc->m_pFile->Seek(0x1D8D, CFile::begin);
	pDoc->m_pFile->Read(&dwActualOffset, 3);

	dwActualOffset = LoROMToSMC(dwActualOffset);

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	pDoc->m_pFile->Read(&wEntranceOffset, 2);

	if(dwActualOffset == 0xD0066)
	{
		// Use the preset number of entrances.
		if(dwOffset == 0xD0060)
		{
			pDoc->m_pFile->Seek(dwOffset + 2, CFile::begin);
			pDoc->m_pFile->Read(&wNextOffset, 2);
			dwLength = wNextOffset - wEntranceOffset;
		}
		else if(dwOffset == 0xD0062)
		{
			pDoc->m_pFile->Seek(dwOffset + 2, CFile::begin);
			pDoc->m_pFile->Read(&wNextOffset, 2);
			dwLength = wNextOffset - wEntranceOffset;
		}
		else if(dwOffset == 0xD0064)
		{
			dwLength = 0x19A - wEntranceOffset;
		}
	}
	else
	{
		// Grab the number of entrances from the ROM.
		if(dwOffset == 0xD0060)
		{
			pDoc->m_pFile->Seek(0xD0066, CFile::begin);
			pDoc->m_pFile->Read(&dwLength, 1);
		}
		else if(dwOffset == 0xD0062)
		{
			pDoc->m_pFile->Seek(0xD0067, CFile::begin);
			pDoc->m_pFile->Read(&dwLength, 1);
		}
		else if(dwOffset == 0xD0064)
		{
			pDoc->m_pFile->Seek(0xD0068, CFile::begin);
			pDoc->m_pFile->Read(&dwLength, 1);
		}

		dwLength *= 5;
	}

	dwActualOffset += wEntranceOffset;

	entrances.SetSize(dwLength / 5);

	pDoc->m_pFile->Seek(dwActualOffset, CFile::begin);
	for(UINT i = 0; i < dwLength / 5; i++)
	{
		entrances[i] = new Entrance;
		pDoc->m_pFile->Read(entrances[i], 5);
	}

	return TRUE;
}

Entrance* WorldData::FindEntranceFromSource(BYTE x, BYTE y)
{
	for(int i = 0; i < entrances.GetSize(); i++)
	{
		Entrance* pEntr = entrances[i];
		if(pEntr->srcX == x && pEntr->srcY == y) return pEntr;
	}

	return NULL;
}

BOOL WorldData::StoreMapData(int nMap)
{
	if(nMap == MAP_OVERWORLD)
	{
		BYTE* rleMap = new BYTE[65536];
		FillMemory(rleMap, 65536, 0xFF);
		WORD wRows[256];
		WORD wPtr = 0;
		
		// Walk through the map, row by row, compressing it with RLE.
		for(int y = 0; y < 256; y++)
		{
			wRows[y] = wPtr;

			BYTE bPrevTile = 0xFF;
			BYTE bTile = 0xFF;
			int rleCount = 0;

			for(int x = 0; x < 256; x++)
			{
				bPrevTile = bTile;
				bTile = map[y * 256 + x];

				if(bTile != bPrevTile)
				{
					if(rleCount == 0)
					{
						rleMap[wPtr++] = bTile;
						if(bTile == 0x00 || bTile == 0x10 || bTile == 0x20 || bTile == 0x30)
							x += 3;
					}
					else
					{
						rleMap[wPtr - 1] = bPrevTile + 0x80;
						rleMap[wPtr++] = rleCount;
						rleMap[wPtr++] = bTile;
						if(bTile == 0x00 || bTile == 0x10 || bTile == 0x20 || bTile == 0x30)
							x += 3;

						rleCount = 0;
					}
				}
				else
				{
					rleCount++;
				}
			}

			if(rleCount)
			{
				rleMap[wPtr - 1] = bTile + 0x80;
				rleMap[wPtr++] = rleCount;
				rleCount = 0;
			}

			rleMap[wPtr++] = 0xFF;
		}

		// If the size of the map is less than 0x4000 bytes, it will fit
		// in the area already used by the ROM (no expansion necessary).
		// Otherwise, begin storing the map at 0x100200.
		if(wPtr < 0x4000)
		{
			DWORD dwPtr = 0xB0680;

			DWORD dwPtr0 = SMCToLoROM(dwPtr);
			DWORD dwPtr1 = SMCToLoROM(dwPtr + 1);

			pDoc->m_pFile->Seek(0x7CFB, CFile::begin);		// LDA Moon, X
			pDoc->m_pFile->Write(&dwPtr0, 3);
			pDoc->m_pFile->Seek(0x7D04, CFile::begin);		// LDA Moon + 1, X
			pDoc->m_pFile->Write(&dwPtr1, 3);

			pDoc->m_pFile->Seek(0xB0200, CFile::begin);
			pDoc->m_pFile->Write(&wRows, 512);

			pDoc->m_pFile->Seek(dwPtr, CFile::begin);
			pDoc->m_pFile->Write(rleMap, 0x4000);
		}
		else
		{
			// Clear out the old data.
			BYTE* pClearData = new BYTE[0x4000];
			FillMemory(pClearData, 0x4000, 0xFF);
			pDoc->m_pFile->Seek(0xB0680, CFile::begin);
			pDoc->m_pFile->Write(pClearData, 0x4000);

			DWORD dwPtr = pDoc->m_dwExpansionOffset; //0x100200;

			DWORD dwPtr0 = SMCToLoROM(dwPtr);
			DWORD dwPtr1 = SMCToLoROM(dwPtr + 1);

			pDoc->m_pFile->Seek(0x7CFB, CFile::begin);		// LDA Overworld, X
			pDoc->m_pFile->Write(&dwPtr0, 3);
			pDoc->m_pFile->Seek(0x7D04, CFile::begin);		// LDA Overworld + 1, X
			pDoc->m_pFile->Write(&dwPtr1, 3);

			pDoc->m_pFile->Seek(0xB0200, CFile::begin);
			pDoc->m_pFile->Write(&wRows, 512);

			pDoc->m_pFile->Seek(dwPtr, CFile::begin);
			pDoc->m_pFile->Write(rleMap, 0x10000);

			// We've used 64K.
			pDoc->m_dwExpansionOffset += 0x10000;
		}

		delete rleMap;
	}
	else if(nMap == MAP_UNDERGROUND)
	{
		BYTE* rleMap = new BYTE[65536];
		FillMemory(rleMap, 65536, 0xFF);
		WORD wRows[256];
		WORD wPtr = 0;
		
		// Walk through the map, row by row, compressing it with RLE.
		for(int y = 0; y < 256; y++)
		{
			wRows[y] = wPtr;

			BYTE bPrevTile = 0xFF;
			BYTE bTile = 0xFF;
			int rleCount = 0;

			for(int x = 0; x < 256; x++)
			{
				bPrevTile = bTile;
				bTile = map[y * 256 + x];

				if(bTile != bPrevTile)
				{
					if(rleCount == 0)
					{
						rleMap[wPtr++] = bTile;
					}
					else
					{
						rleMap[wPtr - 1] = bPrevTile + 0x80;
						rleMap[wPtr++] = rleCount;
						rleMap[wPtr++] = bTile;
						rleCount = 0;
					}
				}
				else
				{
					rleCount++;
				}
			}

			if(rleCount)
			{	
				rleMap[wPtr - 1] = bTile + 0x80;
				rleMap[wPtr++] = rleCount;
				rleCount = 0;
			}

			rleMap[wPtr++] = 0xFF;
		}

		// If the size of the map is less than 0x1D00 bytes, it will fit
		// in the area already used by the ROM (no expansion necessary).
		// Otherwise, begin storing the map at 0x110200.
		if(wPtr < 0x1D00)
		{
			DWORD dwPtr = 0xB4680;

			DWORD dwPtr0 = SMCToLoROM(dwPtr);
			DWORD dwPtr1 = SMCToLoROM(dwPtr + 1);

			pDoc->m_pFile->Seek(0x7D89, CFile::begin);		// LDA Underground, X
			pDoc->m_pFile->Write(&dwPtr0, 3);
			pDoc->m_pFile->Seek(0x7D92, CFile::begin);		// LDA Underground + 1, X
			pDoc->m_pFile->Write(&dwPtr1, 3);

			pDoc->m_pFile->Seek(0xB0400, CFile::begin);
			pDoc->m_pFile->Write(&wRows, 512);

			pDoc->m_pFile->Seek(dwPtr, CFile::begin);
			pDoc->m_pFile->Write(rleMap, 0x1D00);
		}
		else
		{
			// Clear out the old data.
			BYTE* pClearData = new BYTE[0x1D00];
			FillMemory(pClearData, 0x1D00, 0xFF);
			pDoc->m_pFile->Seek(0xB4680, CFile::begin);
			pDoc->m_pFile->Write(pClearData, 0x1D00);

			DWORD dwPtr = pDoc->m_dwExpansionOffset; //0x110200;

			DWORD dwPtr0 = SMCToLoROM(dwPtr);
			DWORD dwPtr1 = SMCToLoROM(dwPtr + 1);

			pDoc->m_pFile->Seek(0x7D89, CFile::begin);		// LDA Underground, X
			pDoc->m_pFile->Write(&dwPtr0, 3);
			pDoc->m_pFile->Seek(0x7D92, CFile::begin);		// LDA Underground + 1, X
			pDoc->m_pFile->Write(&dwPtr1, 3);

			pDoc->m_pFile->Seek(0xB0400, CFile::begin);
			pDoc->m_pFile->Write(&wRows, 512);

			pDoc->m_pFile->Seek(dwPtr, CFile::begin);
			pDoc->m_pFile->Write(rleMap, 0x10000);

			// We've used 64K.
			pDoc->m_dwExpansionOffset += 0x10000;
		}

		delete rleMap;
	}
	else if(nMap == MAP_MOON)
	{
		BYTE* rleMap = new BYTE[65536];
		FillMemory(rleMap, 65536, 0xFF);
		WORD wRows[256];
		WORD wPtr = 0;
		
		// Walk through the map, row by row, compressing it with RLE.
		for(int y = 0; y < 64; y++)
		{
			wRows[y] = wPtr;

			BYTE bPrevTile = 0xFF;
			BYTE bTile = 0xFF;
			int rleCount = 0;

			for(int x = 0; x < 64; x++)
			{
				bPrevTile = bTile;
				bTile = map[y * 64 + x];

				if(bTile != bPrevTile)
				{
					if(rleCount == 0)
					{
						rleMap[wPtr++] = bTile;
					}
					else
					{
						rleMap[wPtr - 1] = bPrevTile + 0x80;
						rleMap[wPtr++] = rleCount;
						rleMap[wPtr++] = bTile;
						rleCount = 0;
					}
				}
				else
				{
					rleCount++;
				}
			}

			if(rleCount)
			{
				rleMap[wPtr - 1] = bTile + 0x80;
				rleMap[wPtr++] = rleCount;
				rleCount = 0;
			}

			rleMap[wPtr++] = 0xFF;
		}

		// If the size of the map is less than 0xA00 bytes, it will fit
		// in the area already used by the ROM (no expansion necessary).
		// Otherwise, begin storing the map at 0x120200.
		if(wPtr < 0xA00)
		{
			DWORD dwPtr = 0xB6380;

			DWORD dwPtr0 = SMCToLoROM(dwPtr);
			DWORD dwPtr1 = SMCToLoROM(dwPtr + 1);

			pDoc->m_pFile->Seek(0x7DD6, CFile::begin);		// LDA Moon, X
			pDoc->m_pFile->Write(&dwPtr0, 3);
			pDoc->m_pFile->Seek(0x7DDF, CFile::begin);		// LDA Moon + 1, X
			pDoc->m_pFile->Write(&dwPtr1, 3);

			pDoc->m_pFile->Seek(0xB0600, CFile::begin);
			pDoc->m_pFile->Write(&wRows, 128);

			pDoc->m_pFile->Seek(dwPtr, CFile::begin);
			pDoc->m_pFile->Write(rleMap, 0xA00);
		}
		else
		{
			// Clear out the old data.
			BYTE* pClearData = new BYTE[0xA00];
			FillMemory(pClearData, 0xA00, 0xFF);
			pDoc->m_pFile->Seek(0xB6380, CFile::begin);
			pDoc->m_pFile->Write(pClearData, 0xA00);

			DWORD dwPtr = pDoc->m_dwExpansionOffset; //0x120200;

			DWORD dwPtr0 = SMCToLoROM(dwPtr);
			DWORD dwPtr1 = SMCToLoROM(dwPtr + 1);

			pDoc->m_pFile->Seek(0x7DD6, CFile::begin);		// LDA Moon, X
			pDoc->m_pFile->Write(&dwPtr0, 3);
			pDoc->m_pFile->Seek(0x7DDF, CFile::begin);		// LDA Moon + 1, X
			pDoc->m_pFile->Write(&dwPtr1, 3);

			pDoc->m_pFile->Seek(0xB0600, CFile::begin);
			pDoc->m_pFile->Write(&wRows, 128);

			pDoc->m_pFile->Seek(dwPtr, CFile::begin);
			pDoc->m_pFile->Write(rleMap, 0x1000);

			// We've used 64K.  (Well, not -yet-.  I only used 4K, but if I
			// ever make the moon 256x256, 64K will be necessary).
			pDoc->m_dwExpansionOffset += 0x10000;
		}

		delete rleMap;
	}

	return TRUE;
}

BOOL WorldData::StoreEnemyLayout(DWORD dwOffset, DWORD dwLength)
{
	// Extract the tiles (as an array of BYTEs) from the ROM.  They are stored
	// in a chunky sequential 4 bits per pixel format.  The tile data is 8192 
	// bytes long.

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	pDoc->m_pFile->Write(enemyLayout, dwLength);

	return TRUE;
}

BOOL WorldData::StoreEnemyOdds(DWORD dwOffset, DWORD dwLength)
{
	// Extract the tiles (as an array of BYTEs) from the ROM.  They are stored
	// in a chunky sequential 4 bits per pixel format.  The tile data is 8192 
	// bytes long.

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	pDoc->m_pFile->Write(enemyOdds, dwLength);

	return TRUE;
}

BOOL WorldData::StoreTileData(DWORD dwOffset)
{
	// Extract the tiles (as an array of BYTEs) from the ROM.  They are stored
	// in a chunky sequential 4 bits per pixel format.  The tile data is 8192 
	// bytes long.

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	pDoc->m_pFile->Write(tiles, 8192);

	return TRUE;
}

BOOL WorldData::StoreTileFormations(DWORD dwOffset)
{
	// Extract the information telling the game how to combine the various
	// 8x8 tiles to form 16x16 tiles used on the map.  This is useful because
	// it allows identical 8x8 tiles to be reused in different 16x16 tiles,
	// conserving memory.  There are 128 16x16 tiles formed from 4 8x8 tiles,
	// making a total of 512 bytes.

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	pDoc->m_pFile->Write(&tileFormations, 512);

	return TRUE;
}

BOOL WorldData::StoreTileProperties(DWORD dwOffset)
{
	// Extract the information assigning various properties for each of the
	// 128 16x16 tiles.  This information tells the game engine which tiles
	// can be walked on, whether enemy encounters are activated, among other
	// things.  Each property is represented by a WORD, totalling 256 bytes
	// of data for 128 tiles.

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	pDoc->m_pFile->Write(&tileProperties, 256);

	return TRUE;
}

BOOL WorldData::StoreTilePalettes(DWORD dwOffset)
{
	// Extract the data associating each 8x8 tile with one of the 16 color
	// palettes.  Each entry is one byte, for each of the 256 tiles, so
	// there are naturally 256 bytes of data.

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	pDoc->m_pFile->Write(&tilePalettes, 256);

	return TRUE;
}

BOOL WorldData::StorePalettes(DWORD dwOffset)
{
	// Extract the actual color information from the ROM.  There are 64 colors
	// per map, each color being represented by a WORD in a 555 BGR format.
	// This makes a total of 128 bytes per map.

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	pDoc->m_pFile->Write(&palette, 128);

	return TRUE;
}

BOOL WorldData::StoreEntrances(DWORD dwOffset, BOOL bExpand)
{
	// Create a new instance of an Entrance class and fill it with the
	// necessary parameters for every entrance on the map (each is 5 bytes).
	
	static WORD wNextOffset;

	DWORD dwActualOffset = 0;
	WORD wEntranceOffset = 0, wLength = 0;

	if(dwOffset == 0xD0060)
	{
		wLength = entrances.GetSize() * 5;
		
		if(bExpand)
		{
			dwActualOffset = pDoc->m_dwExpansionOffset;
			wEntranceOffset = 0x0000;

			BYTE bEntr = entrances.GetSize();
			pDoc->m_pFile->Seek(0xD0066, CFile::begin);
			pDoc->m_pFile->Write(&bEntr, 1);
		}
		else
		{
			dwActualOffset = 0xD0066;
			wEntranceOffset = 0x0000;
			wNextOffset = wLength;
		}
	}
	else if(dwOffset == 0xD0062)
	{
		wLength = entrances.GetSize() * 5;

		if(bExpand)
		{
			dwActualOffset = pDoc->m_dwExpansionOffset;
			wEntranceOffset = 0x0500;

			BYTE bEntr = entrances.GetSize();
			pDoc->m_pFile->Seek(0xD0067, CFile::begin);
			pDoc->m_pFile->Write(&bEntr, 1);
		}
		else
		{
			dwActualOffset = 0xD0066;
			wEntranceOffset = wNextOffset;
			wNextOffset += wLength;
		}
	}
	else if(dwOffset == 0xD0064)
	{
		wLength = entrances.GetSize() * 5;

		if(bExpand)
		{
			dwActualOffset = pDoc->m_dwExpansionOffset;
			wEntranceOffset = 0x0A00;

			// We've used 3.75K.
			pDoc->m_dwExpansionOffset += 0xF00;

			BYTE bEntr = entrances.GetSize();
			pDoc->m_pFile->Seek(0xD0068, CFile::begin);
			pDoc->m_pFile->Write(&bEntr, 1);
		}
		else
		{
			dwActualOffset = 0xD0066;
			wEntranceOffset = wNextOffset;
			wNextOffset += wLength;
		}
	}

	// Eight pointers to repair.  Wow.
	DWORD dwPtr0 = SMCToLoROM(dwActualOffset);
	DWORD dwPtr1 = SMCToLoROM(dwActualOffset + 1);
	DWORD dwPtr2 = SMCToLoROM(dwActualOffset + 2);
	DWORD dwPtr3 = SMCToLoROM(dwActualOffset + 3);
	DWORD dwPtr4 = SMCToLoROM(dwActualOffset + 4);
	pDoc->m_pFile->Seek(0x1D8D, CFile::begin);  pDoc->m_pFile->Write(&dwPtr0, 3);
	pDoc->m_pFile->Seek(0x1D96, CFile::begin);  pDoc->m_pFile->Write(&dwPtr1, 3);
	pDoc->m_pFile->Seek(0x1DA7, CFile::begin);  pDoc->m_pFile->Write(&dwPtr2, 3);
	pDoc->m_pFile->Seek(0x1DAF, CFile::begin);  pDoc->m_pFile->Write(&dwPtr3, 3);
	pDoc->m_pFile->Seek(0x1DEA, CFile::begin);  pDoc->m_pFile->Write(&dwPtr2, 3);
	pDoc->m_pFile->Seek(0x1DF1, CFile::begin);  pDoc->m_pFile->Write(&dwPtr3, 3);
	pDoc->m_pFile->Seek(0x1DFA, CFile::begin);  pDoc->m_pFile->Write(&dwPtr4, 3);
	pDoc->m_pFile->Seek(0x1E01, CFile::begin);  pDoc->m_pFile->Write(&dwPtr3, 3);

	pDoc->m_pFile->Seek(dwOffset, CFile::begin);
	pDoc->m_pFile->Write(&wEntranceOffset, 2);

	if(bExpand)
	{
		// Clear out the old data.
		BYTE* bEmpty = new BYTE[0x500];
		ZeroMemory(bEmpty, 0x500);
		
		pDoc->m_pFile->Seek(dwActualOffset + wEntranceOffset, CFile::begin);
		pDoc->m_pFile->Write(bEmpty, 0x500);

		delete bEmpty;
	}

	pDoc->m_pFile->Seek(dwActualOffset + wEntranceOffset, CFile::begin);
	for(int i = 0; i < entrances.GetSize(); i++)
	{
		pDoc->m_pFile->Write(entrances[i], 5);
	}

	return TRUE;
}

//////////////////////////////////////////////////////////////////////////

LocationData::LocationData()
{
	objectType = ObjLocationData;
}

LocationData::~LocationData()
{
	for(int i = 0; i < entrances.GetSize(); i++)
		delete entrances[i];
}

Entrance* LocationData::FindEntranceFromSource(BYTE x, BYTE y)
{
	for(int i = 0; i < entrances.GetSize(); i++)
	{
		Entrance* pEntr = entrances[i];
		if(pEntr->srcX == x && pEntr->srcY == y) return pEntr;
	}

	return NULL;
}

Enemy::Enemy()
{
	size.cx = size.cy = 0;
	gold = 0;
	experience = 0;
	hp = 0;
	tiles = NULL;
}

Enemy::~Enemy()
{
	if(tiles) delete tiles;
}

Event::Event()
{
	size = 0;
}

Event::~Event()
{
	for(int i = 0; i < code.GetSize(); i++)
		delete code[i];
}

GlobalLocationData::GlobalLocationData()
{
	pFile = NULL;
}

GlobalLocationData::~GlobalLocationData()
{
	for(int i = 0; i < 256; i++) delete maps[i];
	for(i = 0; i < 16; i++) delete tiles[i];
}

Item::Item()
{
}

Item::~Item()
{
}

Character::Character()
{
}

Character::~Character()
{
}

