// PropertyTree.h : header file
//

#ifndef __PROPERTYTREE_H__
#define __PROPERTYTREE_H__

// Types of items available
#define PROPT_GROUP			0		// used internally
#define PROPT_STATIC		1
#define PROPT_EDIT			2
#define PROPT_NUMEDIT		3
#define PROPT_BOOLEAN		4
#define PROPT_COMBO			5
#define PROPT_COLOR			6
#define PROPT_SYSCOLOR		7
#define PROPT_FILE			8
#define PROPT_FONT			9
#define PROPT_CUSTOM		0xFF

#define PROPF_PREFIXVALUE	0x0001
#define PROPF_OWNERDRAW		0x0002
#define PROPF_SHOWICON		0x0004

#define PROP_DONOTCHANGE	0xFF000000

#define PROPTN_PROPCHANGED	2
#define PROPTN_MEASUREITEM	3
#define PROPTN_DRAWITEM		4

/////////////////////////////////////////////////////////////////////////////
// CPropertyTree window

typedef struct __PROPERTYNODE
{
	UINT id;
	UINT type;
	UINT flags;
	CString name;
	LPARAM lData;
	LPARAM lData2;
	LPARAM lEmpty;
	CString strText;
} PROPERTYNODE, *LPPROPERTYNODE;

typedef struct __PROPERTYITEM
{
	DWORD value;
	LPCTSTR desc;
	LPARAM lData;
} PROPERTYITEM, *LPPROPERTYITEM;

typedef struct tagNMPROPTREE
{
    NMHDR       hdr;
    LPPROPERTYNODE node;
	LPMEASUREITEMSTRUCT lpmis;
	LPDRAWITEMSTRUCT lpdis;
} NMPROPTREE, FAR *LPNMPROPTREE;

#include "PropTreeInLineLB.h"
#include "PropTreeInLineEdit.h"

class CPropertyTree : public CTreeCtrl
{
// Construction
public:
	CPropertyTree();

// Attributes
public:
	HTREEITEM IndexToItem(UINT id);

protected:
	BOOL m_bUsedButton;
	HTREEITEM m_htiSel;
	CImageList m_imgButtons;
	DWORD m_nNameWidth;

	CFont regFont;
	CFont boldFont;

	CString m_strTrue;
	CString m_strFalse;

// Operations
public:
	void AddGroup(UINT id, LPCTSTR name);
	void AddItem(UINT parent, UINT id, LPCTSTR name, UINT type, DWORD flags = 0,
		LPARAM lData = NULL, LPARAM lData2 = NULL, LPARAM lEmpty = NULL);
	void SetItem(UINT id, LPARAM lData, LPARAM lData2 = NULL);

	HTREEITEM GetNextItem(HTREEITEM hItem);
	HTREEITEM GetFirstSiblingItem(HTREEITEM hti);

	void ExpandBranch(HTREEITEM hti);
	void ExpandAll();

	BOOL DeleteAllItems();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CPropertyTree)
	protected:
	virtual void PreSubclassWindow();
	//}}AFX_VIRTUAL

// Implementation
public:
	void SetTrueFalseText(CString strTrue, CString strFalse);
	virtual ~CPropertyTree();

	// Generated message map functions
protected:
	void SendNotification(UINT code, LPPROPERTYNODE node);
	//{{AFX_MSG(CPropertyTree)
	afx_msg void OnSelChanged(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnDestroy();
	//}}AFX_MSG
	afx_msg void OnCustomDraw(NMHDR* pNMHDR, LRESULT* pResult);

	DECLARE_MESSAGE_MAP()

protected:
	COLORREF	m_clr3DFace;
	COLORREF	m_clr3DShadow;
	COLORREF	m_clr3DDkShadow;
	COLORREF	m_clr3DLight;
	COLORREF	m_clr3DHilight;
	COLORREF	m_clrSelBkColor;
	COLORREF	m_clrSelTextColor;

};

/////////////////////////////////////////////////////////////////////////////

#endif