// WorldEnemyView.cpp : implementation file
//

#include "stdafx.h"
#include "FF4Edit.h"
#include "FF4Doc.h"
#include "WorldEnemyView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWorldEnemyView

IMPLEMENT_DYNCREATE(CWorldEnemyView, CWorldBaseView)

CWorldEnemyView::CWorldEnemyView()
{
	m_ptCurrentRegion = CPoint(0, 0);
}

CWorldEnemyView::~CWorldEnemyView()
{
}

BEGIN_MESSAGE_MAP(CWorldEnemyView, CWorldBaseView)
	//{{AFX_MSG_MAP(CWorldEnemyView)
	ON_WM_LBUTTONDOWN()
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
	ON_CBN_SELENDOK(1000, OnChangeEnemySet)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWorldEnemyView drawing

void CWorldEnemyView::OnDraw(CDC* pDC)
{
	CFF4Doc* pDoc = GetDocument();
	CRect clientRect;  GetClientRect(&clientRect);
	int width = m_rcUpdate.Width();
	int height = m_rcUpdate.Height();

	DMSHADERECT dmsr[2];
	dmsr[0].rect.left = m_ptCurrentRegion.x * 32;
	dmsr[0].rect.top = m_ptCurrentRegion.y * 32;
	dmsr[0].rect.right = (m_ptCurrentRegion.x + 1) * 32;
	dmsr[0].rect.bottom = (m_ptCurrentRegion.y + 1) * 32;
	dmsr[0].color = DDColor(31, 31, 31);
	dmsr[0].opacity = 2;

	::SetRectEmpty(&dmsr[1].rect);

	BeginDraw();
	DrawMap(DM_ShadeRects, (DWORD)&dmsr,
			TAG_END, 0);

	int xBegin = 0, xEnd = 0, yBegin = 0, yEnd = 0, xTileBegin = 0, yTileBegin = 0;

	xTileBegin = GetScrollPosition().x;
	yTileBegin = GetScrollPosition().y;
	xBegin = m_rcUpdate.left / 16 + GetScrollPosition().x;
	yBegin = m_rcUpdate.top / 16 + GetScrollPosition().y;
	xEnd = min(xBegin + width / 16 + 1, m_worldData->size.cx);
	yEnd = min(yBegin + height / 16 + 1, m_worldData->size.cy);

	for(int x = xBegin; x < xEnd; x++)
	{
		if(x % 32 == 0)
		{
			VerticalLine((x - xBegin) * m_nRoundDown, 0,
				min(clientRect.bottom, m_worldData->size.cy * m_nRoundDown),
				DDColor(24, 24, 24));
			VerticalLine((x - xBegin) * m_nRoundDown + 1, 0,
				min(clientRect.bottom, m_worldData->size.cy * m_nRoundDown),
				DDColor(24, 24, 24));
		}
	}

	for(int y = yBegin; y < yEnd; y++)
	{
		if(y % 32 == 0)
		{
			HorizontalLine(0, (y - yBegin) * m_nRoundDown,
				min(clientRect.right, m_worldData->size.cx * m_nRoundDown),
				DDColor(24, 24, 24));
			HorizontalLine(0, (y - yBegin) * m_nRoundDown + 1,
				min(clientRect.right, m_worldData->size.cx * m_nRoundDown),
				DDColor(24, 24, 24));
		}
	}

	EndDraw();

	m_wndEnemySets.Invalidate();

	CRect controlRect, boundsRect;
	boundsRect = GetVisibleTileBounds();
	CString label;

	CClientDC dc(this);
	CFont* oldFont = dc.SelectObject(&m_fontBold);
	dc.SetTextColor(::GetSysColor(COLOR_WINDOWTEXT));
	dc.SetBkColor(::GetSysColor(COLOR_WINDOW));

	for(y = 0; y < m_worldData->size.cy / 32; y++)
	{
		for(x = 0; x < m_worldData->size.cx / 32; x++)
		{
			controlRect.left = x * 32 * m_nRoundDown - boundsRect.left * m_nRoundDown + 16;
			controlRect.top = y * 32 * m_nRoundDown - boundsRect.top * m_nRoundDown + 16;
			controlRect.right = (x + 1) * 32 * m_nRoundDown - boundsRect.left * m_nRoundDown - 16;
			controlRect.bottom = controlRect.top + 21;
			dc.FillSolidRect(&controlRect, ::GetSysColor(COLOR_WINDOW));

			controlRect.left += 4;
			label.Format("Enemy Set %d", m_worldData->enemyLayout[y * m_worldData->size.cx / 32 + x]);
			dc.DrawText(label, &controlRect, DT_SINGLELINE | DT_VCENTER);

			controlRect.left = x * 32 * m_nRoundDown - boundsRect.left * m_nRoundDown + 16;
			controlRect.top = y * 32 * m_nRoundDown - boundsRect.top * m_nRoundDown + 40;
			controlRect.right = (x + 1) * 32 * m_nRoundDown - boundsRect.left * m_nRoundDown - 16;
			controlRect.bottom = controlRect.top + 21;
			dc.FillSolidRect(&controlRect, ::GetSysColor(COLOR_WINDOW));

			controlRect.left += 4;
			label.Format("Probability of attack: %d in 255", m_worldData->enemyOdds[y * m_worldData->size.cx / 32 + x]);
			dc.DrawText(label, &controlRect, DT_SINGLELINE | DT_VCENTER);
		}
	}

	dc.SelectObject(oldFont);
}

/////////////////////////////////////////////////////////////////////////////
// CWorldEnemyView diagnostics

#ifdef _DEBUG
void CWorldEnemyView::AssertValid() const
{
	CWorldBaseView::AssertValid();
}

void CWorldEnemyView::Dump(CDumpContext& dc) const
{
	CWorldBaseView::Dump(dc);
}

CFF4Doc* CWorldEnemyView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CFF4Doc)));
	return (CFF4Doc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CWorldEnemyView message handlers

void CWorldEnemyView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CFF4Doc* pDoc = GetDocument();

	if(lHint == NULL || (WorldData*)lHint == m_worldData) return;

	m_worldData = (WorldData*)lHint;

	CSize sizeTotal, sizePage;
	sizeTotal.cx = m_worldData->size.cx;
	sizeTotal.cy = m_worldData->size.cy;
	sizePage.cx = sizePage.cy = 16;
	SetScrollSizes(sizeTotal, sizePage);
	ScrollToPosition(m_worldData->ppdLastScrollPos);

	Invalidate();
}

void CWorldEnemyView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	int xTile, yTile, xScreenTile, yScreenTile;
	CRect clientRect;  GetClientRect(clientRect);

	xTile = GetScrollPosition().x + point.x / 16;
	if(xTile < 0) xTile = 0;
	else if(xTile > m_worldData->size.cx - 1) xTile = m_worldData->size.cx - 1;
	
	yTile = GetScrollPosition().y + point.y / 16;
	if(yTile < 0) yTile = 0;
	else if(yTile > m_worldData->size.cy - 1) yTile = m_worldData->size.cy - 1;

	xScreenTile = point.x / 16;
	if(xScreenTile < 0) xScreenTile = 0;
	else if(xScreenTile > clientRect.Width() / 16) xScreenTile = clientRect.Width() / 16;

	yScreenTile = point.y / 16;
	if(yScreenTile < 0) yScreenTile = 0;
	else if(yScreenTile > clientRect.Height() / 16) yScreenTile = clientRect.Height() / 16;

	if(m_ptCurrentRegion != CPoint(xTile / 32, yTile / 32))
	{
		m_ptCurrentRegion.x = xTile / 32;
		m_ptCurrentRegion.y = yTile / 32;
		Invalidate();

		CRect controlRect, boundsRect;
		boundsRect = GetVisibleTileBounds();

		controlRect.left = m_ptCurrentRegion.x * 32 * m_nRoundDown - boundsRect.left * m_nRoundDown + 16;
		controlRect.top = m_ptCurrentRegion.y * 32 * m_nRoundDown - boundsRect.top * m_nRoundDown + 16;
		controlRect.right = (m_ptCurrentRegion.x + 1) * 32 * m_nRoundDown - boundsRect.left * m_nRoundDown - 16;
		controlRect.bottom = controlRect.top + 256;
		m_wndEnemySets.MoveWindow(&controlRect);
		m_wndEnemySets.SetCurSel(m_worldData->enemyLayout[m_ptCurrentRegion.y * m_worldData->size.cx / 32 + m_ptCurrentRegion.x]);

		controlRect.left = m_ptCurrentRegion.x * 32 * m_nRoundDown - boundsRect.left * m_nRoundDown + 16;
		controlRect.top = m_ptCurrentRegion.y * 32 * m_nRoundDown - boundsRect.top * m_nRoundDown + 40;
		controlRect.right = (m_ptCurrentRegion.x + 1) * 32 * m_nRoundDown - boundsRect.left * m_nRoundDown - 16;
		controlRect.bottom = controlRect.top + 21;
		m_wndEnemyOdds.MoveWindow(&controlRect);
		m_wndEnemyOdds.SetPos(m_worldData->enemyOdds[m_ptCurrentRegion.y * m_worldData->size.cx / 32 + m_ptCurrentRegion.x]);
	}

	SetFocus();

	CWorldBaseView::OnLButtonDown(nFlags, point);
}

int CWorldEnemyView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWorldBaseView::OnCreate(lpCreateStruct) == -1)
		return -1;

	CClientDC dc(this);
	int nHeight = -((dc.GetDeviceCaps(LOGPIXELSY) * 8.5) / 72);
	m_font.CreateFont(nHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_CHARACTER_PRECIS,
		CLIP_CHARACTER_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, _T("Tahoma"));
	m_fontBold.CreateFont(nHeight, 0, 0, 0, FW_BOLD, 0, 0, 0, DEFAULT_CHARSET, OUT_CHARACTER_PRECIS,
		CLIP_CHARACTER_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, _T("Tahoma"));

	m_wndEnemySets.Create(WS_CHILD | WS_VISIBLE | CBS_DROPDOWNLIST, CRect(0, 0, 0, 0), this, 1000);
	m_wndEnemySets.SetFont(&m_fontBold);

	CString s;
	for(int i = 0; i < 13; i++)
	{
		s.Format("Enemy Set %d", i);
		m_wndEnemySets.AddString(s);
	}

	m_wndEnemyOdds.Create(WS_CHILD | WS_VISIBLE | TBS_HORZ | TBS_AUTOTICKS, CRect(0, 0, 0, 0), this, 1001);
	m_wndEnemyOdds.SetTicFreq(16);
	m_wndEnemyOdds.SetRange(0, 256);

	return 0;
}

BOOL CWorldEnemyView::OnScroll(UINT nScrollCode, UINT nPos, BOOL bDoScroll) 
{
	BOOL res = CWorldBaseView::OnScroll(nScrollCode, nPos, bDoScroll);

	CRect controlRect, boundsRect;
	boundsRect = GetVisibleTileBounds();

	controlRect.left = m_ptCurrentRegion.x * 32 * m_nRoundDown - boundsRect.left * m_nRoundDown + 16;
	controlRect.top = m_ptCurrentRegion.y * 32 * m_nRoundDown - boundsRect.top * m_nRoundDown + 16;
	controlRect.right = (m_ptCurrentRegion.x + 1) * 32 * m_nRoundDown - boundsRect.left * m_nRoundDown - 16;
	controlRect.bottom = controlRect.top + 256;
	m_wndEnemySets.MoveWindow(&controlRect);

	return res;
}

void CWorldEnemyView::OnChangeEnemySet()
{
	m_worldData->enemyLayout[m_ptCurrentRegion.y * m_worldData->size.cx / 32 + m_ptCurrentRegion.x] =
		m_wndEnemySets.GetCurSel();
}
